/* -*- c++ -*-
 * Copyright (c) 2003-2007 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bulletin_board.h,v 1.37 2007/01/01 03:17:25 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_BULLETIN_BOARD_H_
#define _LIBOCHUSHA_BULLETIN_BOARD_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/a_bone.h"
#include "libochusha/buffer.h"
#include "libochusha/bbs_thread.h"
#include "libochusha/constants.h"
#include "libochusha/hashtable.h"
#include "libochusha/network_agent.h"
#include "libochusha/ref_count.h"
#include "libochusha/sigslot.h"
#include "libochusha/tree.h"
#include "libochusha/uri_utils.h"


#include <vector>


using namespace std;
namespace Ochusha
{
  class BulletinBoard;
  class Configuration;
  class Repository;
  class ThreadPool;


  class ThreadlistUpdater : public RefCount, public has_slots
  {
    friend class BulletinBoard;
    friend class ThreadlistIterator;

  protected:
    BulletinBoard *board;
    URI base_uri;

    Configuration &config;
    Repository &repository;
    smart_ptr<NetworkAgent> agent;

    // updated_listιˤϤlockwrlock()٤
    // updated_listλȻˤϤlockrdlock()٤
    smart_ptr<RWLock> lock;

    vector<smart_ptr<BBSThread> > updated_list;


    // ļΤϹȤ򲿤Ƥʤñ
    // BulletinBoardThreadlistIteratorȤĤλԴ֤Ʊʬ
    // Ƥ롣SubjectTxtParserʤɤȤΤ(֥饹)
    ThreadlistUpdater(BulletinBoard *board, Configuration &config,
		      Repository &repository, NetworkAgent *agent);

    inline void make_alive(BBSThread *thread) { thread->alive = true; }
    inline bool get_alive(BBSThread *thread) { return thread->alive; }
    void recover_thread_list();
    void update_thread_list();
    virtual void finalize();

    Signal0 state_changed;


  public:
    virtual ~ThreadlistUpdater();
  };


  class ThreadlistIterator : public RefCount, public has_slots
  {
  private:
    smart_ptr<ThreadlistUpdater> updater;
    vector<smart_ptr<BBSThread> > &updated_list;
    int index;


    void started(NetworkAgent *agent, RefCount *data);
    void progressed(NetworkAgent *agent, size_t data_read, uintmax_t total_len,
		    RefCount *data);
    void finished(NetworkAgent *agent, RefCount *data);
    void failed(NetworkAgent *agent, const char *reason, RefCount *data);
    void terminated(NetworkAgent *agent, RefCount *data);

    void changed();


  public:
    ThreadlistIterator(ThreadlistUpdater *updater);
    virtual ~ThreadlistIterator();

    // ̾reader lockɬפǡiterationŪlock/unlock
    // ֤ƱϢϴ˱äǤ뤬pthreadǤƱϹ
    // ʤΤǡäʤˡThreadlistIteratorΥ桼ϡ
    // has_more_threads()next_thread()finished()θƤӽФ
    // ĤǥåԤ٤
    inline int lock() { return updater->lock->rdlock(); }
    inline int trylock() { return updater->lock->tryrdlock(); }
    inline int unlock(int lock_id) { return updater->lock->unlock(lock_id); }


    /* ̯semantics*/

    // has_more_threads()ϤλiterateǤBBSThread¸ߤ
    // true֤ͥåȥsubject.txtɤ߹Ǥʤɡ
    // ³Ǥ⤽λɤ߹ƤʬiterateǤ
    // BBSThread¸ߤʤˤfalse֤
    virtual bool has_more_threads();

    // next_thread()ϤλiterateǤBBSThread֤
    // iterateǤBBSThread̵ˤNULL֤
    virtual BBSThread *next_thread();

    // finished()ƤΥФiterationνλƤ
    // true֤(finished() && !has_more_threads())trueξ硢
    // ƤBBSThreaditerationλȤ̣롣
    virtual bool finished();


    // iterationǽˤʤäfinished()η̤Ȥtrue
    // ֤郎äȤȯԤ롣
    Signal0 state_changed;


    // ʲϥι˥ͥåȥȼäˡ
    // ˱ȯԤ륷ʥ롣̩ƱΤ
    // ݤʳ˼꤬ʤΤǡŬ˽
    Signal0 access_started;
    Signal0 access_progressed;
    Signal0 access_finished;
    Signal0 access_failed;
    Signal0 access_terminated;
  };


  enum {
    OCHUSHA_BBS_TYPE_PSEUDO_FOLDER = -5,
    OCHUSHA_BBS_TYPE_PASSIVE_FOLDER = -4,
    OCHUSHA_BBS_TYPE_ACTIVE_FOLDER = -3,
    OCHUSHA_BBS_TYPE_VIRTUAL_BOARD = -2,
    OCHUSHA_BBS_TYPE_UNKNOWN = -1,
    OCHUSHA_BBS_TYPE_2CH = 0,
    OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP,
    OCHUSHA_BBS_TYPE_JBBS,		// ¸ߤΤ
    OCHUSHA_BBS_TYPE_MACHIBBS,
    OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR,
    OCHUSHA_BBS_TYPE_MITINOKU,		// ΤбƤ̤б
    OCHUSHA_BBS_TYPE_2CH_COMPATIBLE,
    OCHUSHA_BBS_TYPE_2CH_HEADLINE,
    OCHUSHA_BBS_TYPE_2CH_BE,		// ΰʪ(BEĤüä)
    OCHUSHA_BBS_TYPE_2CH_RESERVED,
    OCHUSHA_BBS_TYPE_BBSPINK,
    OCHUSHA_BBS_TYPE_2CH_COMPATIBLE_RO	// ɤ߹
  };


  enum {
    NOT_DIRTY = 0,
    MINOR_MODIFICATION = 1,
    URGENT_SAVE_RECOMMENDED = 10000
  };


  class DATFileExplorer;

  class BulletinBoard : public TreeElement
  {
  private:
    friend class ThreadlistUpdater;
    friend class ThreadlistIterator;

    // updater->updated_listthread_list˥ԡ()
    void update_thread_list(ThreadlistUpdater *updater);
    // thread_listupdater->updated_list˥ԡ(ʤӼԻ)
    void recover_thread_list(ThreadlistUpdater *updater);

    // ιƱ˹⡹1åɤޤǤ˸ꡣ
    // Ĥν񤭴Lock::get_global_lock()åǼ롣
    // ĤNULLʾϹǤʤȤ̣롣
    smart_ptr<ThreadlistUpdater> updater;


    enum {
      MAILTO_MODE_MASK = ((1 << 0) | (1 << 1)),
      LAST_POST_SAGE = 1 << 2,
      LAST_POST_BE = 1 << 3,
      LAST_POST_VIEWER = 1 << 4
    };


  protected:
    int bbs_type;		// ǼĤμ
    char *base_url;		// 㤨 "http://pc5.2ch.net/unix/"
    char *last_modified;	// subject.txtɤ߹߻Last-Modified
    char *date;			// subject.txtɤ߹߻Date

    char *cookie;		// Ǹ˼cookie(ͽ)

    char *rule;			// ֤ΤȤϻפʤġ
    int ui_flags;		// ֤ΤȤϻפʤġ
    char *last_name;		// ֤ΤȤϻפʤġ
    char *last_mail;		// ֤ΤȤϻפʤġ

    char *previous_server;	// žȯľλϿ(žõ)

    list<char *> movement_history;	// ž

    ABone a_bone;		// Ѥܡ

    vector<smart_ptr<BBSThread> > thread_list;	// Ƥ륹Υꥹ
    RefCountHashtable thread_table;	// IDʸBBSThread *ʥơ֥

    bool threadlist_read;	// threadlist.xmlɤ߹ߺѤɤ
    int dirtyness;		// threadlist.xml¸ɬפɤ


  public:
    BulletinBoard(time_t tc = 0, time_t tlm = 0);
    BulletinBoard(AttributeTable &attributes, time_t tc, time_t tlm);
    virtual ~BulletinBoard();

    virtual ThreadlistIterator *get_threadlist(Configuration &config,
					       Repository &repository,
					       ThreadPool &pool,
					       NetworkAgentBroker &broker,
					       bool update);
    virtual DATFileExplorer *get_dat_file_explorer(Configuration &config,
						   Repository &repository);

    BBSThread *lookup_bbs_thread_by_id(const char *id);
    BBSThread *lookup_bbs_thread_by_url(const char *url,
					int *from_p = NULL, int *to_p = NULL);

    virtual bool read_threadlist_xml(Repository &repository);
    virtual bool write_threadlist_xml(Repository &repository);

    int how_dirty() const;
    inline int get_dirtyness() { return dirtyness; }
    inline void notify_minor_modification()
    { dirtyness += MINOR_MODIFICATION; }
    inline void notify_major_modification()
    { dirtyness += URGENT_SAVE_RECOMMENDED; }

    BBSThread *thread_new(const char *id, const char *title);

    inline const char *get_base_url() const { return base_url; }
    virtual void set_base_url(const char *base_url);

    inline int get_bbs_type() const { return bbs_type; }
    virtual void set_bbs_type(int type);

    inline list<char *>::const_iterator get_movement_history_begin() const
    { return movement_history.begin(); }
    inline list<char *>::const_iterator get_movement_history_end() const
    { return movement_history.end(); }
    inline list<char *> &get_movement_history() { return movement_history; }
    inline ABone &get_a_bone() { return a_bone; }

    char *get_id() const;
    char *get_bbs_root() const;

    char *get_url(BBSThread *thread, int from = 0, int to = 0) const;


    inline const char *get_last_modified() const { return last_modified; }
    void set_last_modified(const char *value);

    inline const char *get_date() const { return date; }
    void set_date(const char *value);

    inline const char *get_cookie() const { return cookie; }
    void set_cookie(const char *value);

    inline const char *get_rule() const { return rule; }
    void set_rule(const char *value);

    inline MailtoMode get_mailto_mode() const
    { return static_cast<MailtoMode>(ui_flags & MAILTO_MODE_MASK); }
    void set_mailto_mode(MailtoMode mode);

    inline bool has_last_post_sage() const
    { return (ui_flags & LAST_POST_SAGE) != 0; }
    void set_last_post_sage(bool value);

    inline bool has_last_post_be() const
    { return (ui_flags & LAST_POST_BE) != 0; }
    void set_last_post_be(bool value);

    inline bool has_last_post_viewer() const
    { return (ui_flags & LAST_POST_VIEWER) != 0; }
    void set_last_post_viewer(bool value);

    inline const char *get_last_name() const { return last_name; }
    void set_last_name(const char *value);

    inline const char *get_last_mail() const { return last_mail; }
    void set_last_mail(const char *value);

    inline const char *get_previous_server() const { return previous_server; }
    void set_previous_server(const char *value);

    virtual bool is_folder() const { return false; }
    const char *get_element_name() const;
    virtual void read_extended_tree_element(AttributeTable &attributes);
    virtual void write_extended_tree_element(WriteTree &args);

#if 0
    virtual void chase_board_movement(NetworkAgent *agent) = 0;
    virtual void treat_board_movement(Repository &repository,
				      const char *new_url) = 0;
#endif

    static void get_all_bbs_type_list(vector<int> &bbs_type_list);
    static const char *get_bbs_type_description(int type);

    static char *synthesize_active_folder_url(unsigned int sn);
    static unsigned int get_serial_number_from_active_folder_url(const char *url);
  };


  class DATFileExplorer : public ThreadlistUpdater
  {
  public:
    DATFileExplorer(BulletinBoard *board, Configuration &config,
		    Repository &repository);
    virtual ~DATFileExplorer();

    virtual bool explore(ThreadPool &thread_pool) = 0;


    // »ҤDATե򸫤Ĥ٤ȯԤ
    // void dat_file_found(BulletinBoard *board, BBSThread *thread);
    signal2<BulletinBoard *, BBSThread *> dat_file_found;

    // ȯλȯԤ
    // void finished(BulletinBoard *board);
    signal1<BulletinBoard *> finished;
  };
}
#endif

#define _LIBOCHUSHA_BULLETIN_BOARD_DEFINED_	1

#include "libochusha/bbs_thread.h"

#endif	/* _LIBOCHUSHA_BULLETIN_BOARD_H_ */
