/* -*- c++ -*-
 * Copyright (c) 2003-2007 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: misc_utils.cc,v 1.11 2007/01/04 06:30:19 fuyu Exp $
 */

#ifdef __linux__
#define _XOPEN_SOURCE
#endif
#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libochusha/misc_utils.h"

#include <cassert>
#include <cerrno>
#include <climits>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#ifdef HAVE_LOCALE_H
#include <locale.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#ifdef HAVE_TIME_H
#include <time.h>
#endif


/*
 * mkdir_p
 *
 * mkdir -pޥɤΤ褦ˡpathǻꤵ줿ǥ쥯ȥŪ
 * 롣
 * ǥ쥯ȥ꤬¸ߤ뤫ǥ쥯ȥκˤ0
 * ʳξ-1֤
 */
int
mkdir_p(const char *path)
{
  char buffer[PATH_MAX];
  char *cur_pos;

  assert(path != NULL);

  strncpy(buffer, path, PATH_MAX);
  buffer[PATH_MAX - 1] = '\0';
  size_t len = strlen(buffer);
  if (buffer[len - 1] == '/')
    buffer[len - 1] = '\0';	// Ǹ'/'̵롣

  cur_pos = buffer;
  while (1)
    {
      int result;
      struct stat sb;
      char tmp_char;

      cur_pos = strchr(cur_pos, '/');
      if (cur_pos != NULL)
	{
	  tmp_char = *(++cur_pos);
	  *cur_pos = '\0';
	}
      else
	tmp_char = '\0';

      result = stat(buffer, &sb);

      if (result == -1)
	{
	  if (errno != ENOENT)
	    return -1;
	
	  result = mkdir(buffer, 0x1c0);
	  if (result != 0)
	    return -1;
	}
      else
	if (!S_ISDIR(sb.st_mode))
	  return -1;

      if (tmp_char == '\0')
	return 0;

      *cur_pos = tmp_char;
    }
}


namespace Ochusha
{

string &
append_markup_escaped_text(string &str, const char *text)
{
  const char *cur_pos = text;

  while (true)
    {
      const char *tmp_pos = strpbrk(cur_pos, "&<>'\"");

      if (tmp_pos == NULL)
	{
	  str.append(cur_pos);
	  return str;
	}

      if (tmp_pos - cur_pos > 0)
	str.append(cur_pos, tmp_pos - cur_pos);

      switch (*tmp_pos)
	{
	case '&':
	  str.append("&amp;"); break;
	case '<':
	  str.append("&lt;"); break;
	case '>':
	  str.append("&gt;"); break;
	case '\'':
	  str.append("&apos;"); break;
	case '"':
	  str.append("&quot;"); break;
	}
      cur_pos = tmp_pos + 1;
    }
}

}


char *
markup_escape_text(const char *text)
{
  string tmp;
  Ochusha::append_markup_escaped_text(tmp, text);
  return strdup(tmp.c_str());
}


/*
 * Ϥޥɥ饤ȤƻȤ˥פɬפʸ
 * ƥפʸΥԡ֤
 * Ǥϡ#\#֤
 */
char *
shell_escape_text(const char *text)
{
  string command_line;
  const char *cur_pos = text;
  while (*cur_pos != '\0')
    {
      if (*cur_pos != '#')
	command_line.append(cur_pos, 1);
      else
	command_line.append("\\#");
      ++cur_pos;
    }

  return strdup(command_line.c_str());
}


/*
 * get_utc_from_date
 *
 * Ϳ줿ʸɽʸȤƲᤷUTC֤
 */
time_t
get_utc_from_date(const char *date_string)
{
#ifdef HAVE_LOCALE_H
  static char *lc_time = NULL;
#endif
  struct tm tm;
  char *parse_result;

#ifdef HAVE_LOCALE_H
  if (lc_time == NULL)
    {
      lc_time = setlocale(LC_TIME, "C");
      tzset();
      return -1;
    }
#endif

  memset(&tm, 0, sizeof(struct tm));
#if 0
  /*
   * GMTΤ褦timezoneλꤵ줿date_string顢
   * struct tmǤĤäΤ
   * LinuxǤtimezone̵꤬뤵Ƥ͡
   * SolarisǤ%Z"GMT"ȥ顼ˤʤäƤޤ
   */
  parse_result = strptime(date_string, "%a, %d %b %Y %T %Z", &tm);

  return mktime(&tm);
#else
  /*
   * ȡdate_string"GMT"ʳ褿˻郎
   * 2chΥФ˴ؤ¤Linux/SolarisǤԤ褦
   * 郎֤äƤΤǡ괺Ƥ
   */
  parse_result = strptime(date_string, "%a, %d %b %Y %T", &tm);

# ifdef HAVE_TIMEGM
  return timegm(&tm);
# else /* Ķad-hoc塼 */
  return mktime(&tm) + (strstr(date_string, "GMT") != NULL ? 32400 : 0);
# endif
#endif
}


/*
 * Ĺǽstrpbrkؿ
 * stringĹlenʸcharset˴ޤޤǽʸؤΥݥ󥿤֤
 * ʤNULL֤
 */
char *
mempbrk(const char *str, const char *charset, size_t len)
{
  while (len > 0)
    {
      const char *c = charset;
      char s = *str;
      while (*c)
	{
	  if (*c == s)
	    return const_cast<char *>(str);	// strpbrk˹碌
	  c++;
	}
      str++;
      len--;
    }
  return NULL;
}


char *
reverse_strpbrk(const char *str, const char *charset)
{
  const char *s = str;
  while (*s != '\0')
    {
      if (strchr(charset, *s) == NULL)
	return const_cast<char *>(s);
      s++;
    }

  return NULL;
}


#ifndef HAVE_STRNSTR
/*-
 * Copyright (c) 2001 Mike Barcroft <mike@FreeBSD.org>
 * Copyright (c) 1990, 1993
 *      The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Chris Torek.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

char *
strnstr(const char *s, const char *find, size_t slen)
{
  char c, sc;
  size_t len;

  if ((c = *find++) != '\0')
    {
      len = strlen(find);
      do
	{
	  do
	    {
	      if ((sc = *s++) == '\0' || slen-- < 1)
		return NULL;
	    }
	  while (sc != c);

	  if (len > slen)
	    return NULL;
	}
      while (strncmp(s, find, len) != 0);

      s--;
    }
  return const_cast<char *>(s);
}
#endif


#ifndef HAVE_STRDUP
char *
strdup(const char *src)
{
  char *result = (char *)malloc(strlen(src) + 1);
  if (result == NULL)
    return NULL;	// Out of Memory
  strcpy(result, src);
  return result;
}
#endif


#ifndef HAVE_STRNDUP
char *
strndup(const char *src, size_t len)
{
  char *result = (char *)malloc(len + 1);
  if (result == NULL)
    return NULL;	// Out of Memory

  strncpy(result, src, len);
  result[len] = '\0';

  return result;
}
#endif


#ifndef HAVE_STRCASECMP
/*
 * Copyright (c) 1987, 1993
 *      The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
int
strcasecmp(const char *s1, const char *s2)
{
  register const unsigned char *us1 = (const unsigned char *)s1;
  register const unsigned char *us2 = (const unsigned char *)s2;

  while (tolower(*us1) == tolower(*us2++))
    if (*us1++ == '\0')
      return 0;
  return tolower(*us1) - tolower(*--us2);
}
#endif


#ifndef HAVE_STRNCASECMP
int
strncasecmp(const char *s1, const char *s2, size_t n)
{
  if (n != 0)
    {
      register const unsigned char *us1 = (const unsigned char *)s1;
      register const unsigned char *us2 = (const unsigned char *)s2;
      do
	{
	  if (tolower(*us1) != tolower(*us2++))
	    return tolower(*us1) - tolower(*--us2);
	  if (*us1++ == '\0')
	    break;
	}
      while (--n != 0);
    }
  return 0;
}
#endif


# ifndef HAVE_STRCASESTR
char *
strcasestr(const char *big, const char *little)
{
  int c;
  size_t len;
  if ((c = *little++) != '\0')
    {
      c = tolower(c);
      len = strlen(little);
      do
	{
	  char bc;
	  do
	    {
	      if ((bc = *big++) == '\0')
		return NULL;
	    }
	  while (tolower(bc) != c);
	}
      while (strncasecmp(big, little, len) != 0);

      big--;
    }
  return const_cast<char *>(big);
}
# endif


# ifndef HAVE_STRNCASESTR
char *
strncasestr(const char *big, const char *little, size_t blen)
{
  int c;
  size_t len;
  if ((c = *little++) != '\0')
    {
      c = tolower(c);
      len = strlen(little);
      do
	{
	  char bc;
	  do
	    {
	      if ((bc = *big++) == '\0' || --blen < 0)
		return NULL;
	    }
	  while (tolower(bc) != c);

	  if (len > blen)
	    return NULL;
	}
      while (strncasecmp(big, little, len) != 0);

      big--;
    }
  return const_cast<char *>(big);
}
# endif


char *
strnrstr(const char *big, const char *little, size_t blen)
{
  size_t len = strlen(little);
  const char *max_pos = big + blen - len;
  const char *cur_pos = max_pos;

  int c = *little;
  while (big <= cur_pos)
    { /* ޤΨŪǤʤΤϵˤʤ*/
      if (*cur_pos == c && strncmp(cur_pos, little, len) == 0)
	return const_cast<char *>(cur_pos);
      cur_pos--;
    }

  return NULL;
}


char *
strncaserstr(const char *big, const char *little, size_t blen)
{
  size_t len = strlen(little);
  const char *max_pos = big + blen - len;
  const char *cur_pos = max_pos;

  int c = tolower(*little);
  while (big <= cur_pos)
    { // ޤΨŪǤʤΤϵˤʤ
      if (tolower(*cur_pos) == c && strncasecmp(cur_pos, little, len) == 0)
	return const_cast<char *>(cur_pos);
      cur_pos--;
    }

  return NULL;
}
