/*
 * Copyright (c) 2003-2007 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: stock.c,v 1.16 2007/01/01 03:17:39 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#include "libochushagtk/stock.h"

#include <gtk/gtk.h>


#include "libochushagtk/active_folder_16.h"
#include "libochushagtk/add_board_16.h"
#include "libochushagtk/add_bookmark_16.h"
#include "libochushagtk/add_bookmark_24.h"
#include "libochushagtk/board_16.h"
#include "libochushagtk/bookmark_16.h"
#include "libochushagtk/bookmark_folder_16.h"
#include "libochushagtk/edit_disabled_16.h"
#include "libochushagtk/edit_enabled_16.h"
#include "libochushagtk/embeded_editor_16.h"
#include "libochushagtk/folder_closed_16.h"
#include "libochushagtk/folder_open_16.h"
#include "libochushagtk/missing_image_24.h"

#include "libochushagtk/net_done_16.h"
#include "libochushagtk/net_error_16.h"
#include "libochushagtk/net_notice_16.h"
#include "libochushagtk/net_preparation_16.h"
#include "libochushagtk/net_wait_16.h"
#include "libochushagtk/net_progress0_16.h"
#include "libochushagtk/net_progress1_16.h"
#include "libochushagtk/net_progress2_16.h"
#include "libochushagtk/net_progress3_16.h"
#include "libochushagtk/net_progress4_16.h"
#include "libochushagtk/net_progress5_16.h"
#include "libochushagtk/net_progress6_16.h"
#include "libochushagtk/net_progress7_16.h"


#include <stdio.h>


static void
add_stock_icon(GtkIconFactory *factory, const char *stock_id,
	       GdkPixbuf *pixbuf, GtkIconSize size)
{
  GtkIconSource *source = gtk_icon_source_new();
  GtkIconSet *set = gtk_icon_set_new();
  gint pixel_size;

  gtk_icon_source_set_icon_name(source, stock_id);
  gtk_icon_set_add_source(set, source);
  gtk_icon_source_free(source);

  gtk_icon_factory_add(factory, stock_id, set);
  gtk_icon_set_unref(set);

  /* ɥۥåн */
  switch (size)
    {
    case GTK_ICON_SIZE_MENU: pixel_size = 16; break;
    case GTK_ICON_SIZE_SMALL_TOOLBAR: pixel_size = 18; break;
    case GTK_ICON_SIZE_LARGE_TOOLBAR: pixel_size = 24; break;
    case GTK_ICON_SIZE_BUTTON: pixel_size = 20; break;
    case GTK_ICON_SIZE_DND: pixel_size = 32; break;
    case GTK_ICON_SIZE_DIALOG: pixel_size = 48; break;
    default: pixel_size = 0;		/* Τǡ */
    }
  gtk_icon_theme_add_builtin_icon(stock_id, pixel_size, pixbuf);

  g_object_unref(pixbuf);	/* ݤʤΤǤunrefƤޤ */
}


static void
add_stock_icon2(GtkIconFactory *factory, const char *stock_id,
		GdkPixbuf *pixbuf1, gint size1,
		GdkPixbuf *pixbuf2, gint size2)
{
  GtkIconSource *source = gtk_icon_source_new();
  GtkIconSet *set = gtk_icon_set_new();

  gtk_icon_source_set_icon_name(source, stock_id);
  gtk_icon_set_add_source(set, source);

  gtk_icon_source_free(source);

  gtk_icon_factory_add(factory, stock_id, set);

  gtk_icon_set_unref(set);

  gtk_icon_theme_add_builtin_icon(stock_id, size1, pixbuf1);
  gtk_icon_theme_add_builtin_icon(stock_id, size2, pixbuf2);

  g_object_unref(pixbuf1);	/* ݤʤΤǤunrefƤޤ */
  g_object_unref(pixbuf2);	/* ݤʤΤǤunrefƤޤ */
}


static void
ochusha_initialize_icons(void)
{
  GtkIconFactory *factory = gtk_icon_factory_new();

  add_stock_icon(factory, OCHUSHA_STOCK_ACTIVE_FOLDER,
		 gdk_pixbuf_new_from_inline(-1, active_folder_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_ADD_BOARD,
		 gdk_pixbuf_new_from_inline(-1, add_board_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon2(factory, OCHUSHA_STOCK_ADD_BOOKMARK,
		  gdk_pixbuf_new_from_inline(-1, add_bookmark_16,
					     FALSE, NULL), 16,
		  gdk_pixbuf_new_from_inline(-1, add_bookmark_24,
					     FALSE, NULL), 24);

  add_stock_icon(factory, OCHUSHA_STOCK_BOARD,
		 gdk_pixbuf_new_from_inline(-1, board_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_BOOKMARK,
		 gdk_pixbuf_new_from_inline(-1, bookmark_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_BOOKMARK_FOLDER,
		 gdk_pixbuf_new_from_inline(-1, bookmark_folder_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_EDIT_DISABLED,
		 gdk_pixbuf_new_from_inline(-1, edit_disabled_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_EDIT_ENABLED,
		 gdk_pixbuf_new_from_inline(-1, edit_enabled_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_EMBEDED_EDITOR,
		 gdk_pixbuf_new_from_inline(-1, embeded_editor_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_FOLDER_CLOSED,
		 gdk_pixbuf_new_from_inline(-1, folder_closed_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_FOLDER_OPEN,
		 gdk_pixbuf_new_from_inline(-1, folder_open_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_MISSING_IMAGE,
		 gdk_pixbuf_new_from_inline(-1, missing_image_24,
					    FALSE, NULL),
		 GTK_ICON_SIZE_LARGE_TOOLBAR);


  add_stock_icon(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON0,
		 gdk_pixbuf_new_from_inline(-1, net_progress0_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON1,
		 gdk_pixbuf_new_from_inline(-1, net_progress1_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON2,
		 gdk_pixbuf_new_from_inline(-1, net_progress2_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON3,
		 gdk_pixbuf_new_from_inline(-1, net_progress3_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON4,
		 gdk_pixbuf_new_from_inline(-1, net_progress4_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON5,
		 gdk_pixbuf_new_from_inline(-1, net_progress5_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON6,
		 gdk_pixbuf_new_from_inline(-1, net_progress6_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON7,
		 gdk_pixbuf_new_from_inline(-1, net_progress7_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_NET_PREPARATION_ADDON,
		 gdk_pixbuf_new_from_inline(-1, net_preparation_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_NET_DONE_ADDON,
		 gdk_pixbuf_new_from_inline(-1, net_done_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_NET_NOTICE_ADDON,
		 gdk_pixbuf_new_from_inline(-1, net_notice_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_NET_ERROR_ADDON,
		 gdk_pixbuf_new_from_inline(-1, net_error_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);

  add_stock_icon(factory, OCHUSHA_STOCK_NET_WAIT_ADDON,
		 gdk_pixbuf_new_from_inline(-1, net_wait_16,
					    FALSE, NULL),
		 GTK_ICON_SIZE_MENU);


  gtk_icon_factory_add_default(factory);

  g_object_unref(factory);
}


void
ochusha_initialize_stock_items(void)
{
#if 0
  static const GtkStockItem ochusha_stock_items[] = {
    {
      (char *)OCHUSHA_STOCK_ACTIVE_FOLDER,
      (char *)N_("Active Folder"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_ADD_BOARD,
      (char *)N_("Add Board"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_ADD_BOOKMARK,
      (char *)N_("Add Bookmark"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_BOARD,
      (char *)N_("Board"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_BOOKMARK,
      (char *)N_("Bookmark"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_BOOKMARK_FOLDER,
      (char *)N_("Bookmark Folder"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_EDIT_DISABLED,
      (char *)N_("Edit Disabled"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_EDIT_ENABLED,
      (char *)N_("Edit Enabled"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_EMBEDED_EDITOR,
      (char *)N_("Embeded Editor"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_FOLDER_CLOSED,
      (char *)N_("Folder Closed"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_FOLDER_OPEN,
      (char *)N_("Folder Open"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_MISSING_IMAGE,
      (char *)N_("Missing Image"),
      0, 0,
      (char *)PACKAGE
    }
  };
#endif

  ochusha_initialize_icons();

#if 0
  /* ɬפʤʡġġ */
  gtk_stock_add_static(ochusha_stock_items, G_N_ELEMENTS(ochusha_stock_items));
#endif
}
