/* -*- c++ -*-
 * Copyright (c) 2003-2007 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: board_ui.cc,v 1.15 2007/01/01 03:18:25 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "board_ui.h"

#include "controller.h"
#include "threadlist_view.h"

#include "libochusha/ochusha_procedure.h"

#include "libochushagtk/combo_action.h"


#include <cassert>
#include <cstdio>
#include <cstdlib>


namespace Ochusha
{


enum {
  THREADLIST_RULE_LABEL = 0,
  THREADLIST_RULE_NAME,
  THREADLIST_RULE_N_COLUMNS,
};


static GQuark rule_name_id;


BoardUI::BoardUI(Controller &controller)
  : controller(controller),
    action_group(create_action_group())
{
  rule_name_id = g_quark_from_static_string("BoardUI::RuleName");

  g_signal_connect(controller.get_prefs().show_hidden_threads, "toggled",
		   reinterpret_cast<GCallback>(show_hidden_threads_toggled_cb),
		   this);
}


BoardUI::~BoardUI()
{
  g_object_unref(threadlist_rule_model);
  g_object_unref(default_threadlist_rule_model);
  g_object_unref(threadlist_rule);
  g_object_unref(default_threadlist_rule);
  g_object_unref(action_group);
}


static void
set_label_text(GtkMenuItem *item,
	       GtkLabel *widget, GtkTreeModel *model, GtkTreeIter *iter,
	       gpointer unused)
{
  const char *label = NULL;
  gtk_tree_model_get(model, iter, THREADLIST_RULE_LABEL, &label, -1);
  gtk_label_set_text(GTK_LABEL(widget), label);
}


static void
set_combo_text(GtkCellLayout *layout, GtkCellRenderer *renderer,
	       GtkTreeModel *model, GtkTreeIter *iter, gpointer unused)
{
  const char *label = NULL;
  gtk_tree_model_get(model, iter, THREADLIST_RULE_LABEL, &label, -1);
  if (label == NULL)
    label = _("Implementation Error?");

  g_object_set(renderer, "text", label, NULL);
}


GtkActionGroup *
BoardUI::create_action_group()
{
  static GtkActionEntry entries[] = {
    {
      "NewThread", GTK_STOCK_NEW,
      N_("_New Thread"), "",
      N_("New Thread"),
      reinterpret_cast<GCallback>(new_thread_cb)
    },
    {
      "BoardProperties", GTK_STOCK_PROPERTIES,
      N_("Board _Properties"), NULL,
      N_("Board Properties"),
      reinterpret_cast<GCallback>(board_properties_cb)
    }
  };

  action_group = gtk_action_group_new("BoardActions");
  gtk_action_group_set_translation_domain(action_group, PACKAGE_NAME);
  gtk_action_group_add_actions(action_group, entries,
			       G_N_ELEMENTS(entries), this);

  new_thread = gtk_action_group_get_action(action_group, "NewThread");
  gtk_action_set_sensitive(new_thread, FALSE);

  board_properties = gtk_action_group_get_action(action_group,
						 "BoardProperties");


  GtkListStore *store = gtk_list_store_new(THREADLIST_RULE_N_COLUMNS,
					   G_TYPE_POINTER,
					   G_TYPE_POINTER);
  default_threadlist_rule_model = GTK_TREE_MODEL(store);

  OchushaComboAction *combo_action
    = ochusha_combo_action_new_with_model("DefaultThreadlistRule",
					  _("Default Threadlist _Rule"),
					  _("Default Threadlist Rule"),
					  NULL,
					  default_threadlist_rule_model);
  ochusha_combo_action_set_menu_data_func(combo_action,
					  set_label_text, this, NULL);
  ochusha_combo_action_set_cell_data_func(combo_action,
					  set_combo_text, this, NULL);
  g_signal_connect(combo_action, "changed",
		   reinterpret_cast<GCallback>(default_threadlist_rule_changed_cb),
		   this);

  default_threadlist_rule = GTK_ACTION(combo_action);
  gtk_action_group_add_action(action_group, default_threadlist_rule);


  store = gtk_list_store_new(THREADLIST_RULE_N_COLUMNS,
			     G_TYPE_POINTER,
			     G_TYPE_POINTER);
  threadlist_rule_model = GTK_TREE_MODEL(store);

  combo_action = ochusha_combo_action_new_with_model("ThreadlistRule",
						     _("Current Threadlist _Rule"),
						     _("Current Threadlist Rule"),
						     NULL,
						     threadlist_rule_model);
  ochusha_combo_action_set_menu_data_func(combo_action,
					  set_label_text, this, NULL);
  ochusha_combo_action_set_cell_data_func(combo_action,
					  set_combo_text, this, NULL);
  g_signal_connect(combo_action, "changed",
		   reinterpret_cast<GCallback>(threadlist_rule_changed_cb),
		   this);

  threadlist_rule = GTK_ACTION(combo_action);
  gtk_action_group_add_action(action_group, threadlist_rule);


  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     action_group, 0);
  return action_group;
}


void
BoardUI::new_thread_cb(GtkAction *action, BoardUI *ui)
{
#warning "Implement This!"
  fprintf(stderr, "BoardUI::new_thread_cb(): Why this is called?\n");
}


void
BoardUI::board_properties_cb(GtkAction *action, BoardUI *ui)
{
  BulletinBoard *board = ui->controller.get_view_manager().get_current_board();
  if (board != NULL)
    ui->controller.get_boardlist_ui().open_properties(board);
}


void
BoardUI::prepare_session()
{
}


GtkWidget *
BoardUI::create_board_view(BulletinBoard *board, TabLabel *label)
{
  GtkWidget *view = ThreadlistView::create(controller, board, label);
  GtkWidget *widget = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(widget),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_widget_show(view);
  gtk_container_add(GTK_CONTAINER(widget), view);
  return widget;
}


void
BoardUI::notify_current_board_view(GtkWidget *widget, BulletinBoard *board)
{
  if (board == NULL)
    {
#if 0
      gtk_action_set_sensitive(new_thread, FALSE);
#endif
      gtk_action_set_sensitive(board_properties, FALSE);
      gtk_action_set_sensitive(threadlist_rule, FALSE);
      set_current_threadlist_rule(NULL);
    }
  else
    {
#if 0
      gtk_action_set_sensitive(new_thread, TRUE);
#endif
      gtk_action_set_sensitive(board_properties, TRUE);
      gtk_action_set_sensitive(threadlist_rule, TRUE);
      set_current_threadlist_rule(board->get_rule());
    }
}


void
BoardUI::default_threadlist_rule_changed_cb(OchushaComboAction *action,
					    GtkTreeModel *model,
					    GtkTreeIter *iter,
					    GtkTreePath *path,
					    BoardUI *ui)
{
  const char *name = NULL;
  gtk_tree_model_get(model, iter, THREADLIST_RULE_NAME, &name, -1);

  g_return_if_fail(name != NULL);
  ui->set_default_threadlist_rule(name);
}


void
BoardUI::threadlist_rule_changed_cb(OchushaComboAction *action,
				    GtkTreeModel *model,
				    GtkTreeIter *iter,
				    GtkTreePath *path,
				    BoardUI *ui)
{
  const char *name = NULL;
  gtk_tree_model_get(model, iter, THREADLIST_RULE_NAME, &name, -1);

  BulletinBoard *board = ui->controller.get_view_manager().get_current_board();
  if (board != NULL)
    {
      const char *previous_rule = board->get_rule();
      if (previous_rule == NULL)
	previous_rule
	  = ui->controller.get_prefs().get_default_threadlist_rule();
      const char *effective_rule
	= name != NULL
	? name : ui->controller.get_prefs().get_default_threadlist_rule();

      bool redraw = strcmp(previous_rule, effective_rule) != 0;

      board->set_rule(name);
      if (redraw)
	ui->controller.refresh(board, false);
    }

  ui->set_current_threadlist_rule(name);
}


void
BoardUI::set_current_threadlist_rule(const char *name)
{
  GtkTreeIter iter;
  gboolean result = gtk_tree_model_get_iter_first(threadlist_rule_model, &iter);

  if (result)
    {
      if (name != NULL)
	{
	  result = FALSE;
	  while (gtk_tree_model_iter_next(threadlist_rule_model, &iter))
	    {
	      const char *rule_name = NULL;
	      gtk_tree_model_get(threadlist_rule_model, &iter,
				 THREADLIST_RULE_NAME, &rule_name, -1);
	      g_return_if_fail(rule_name != NULL);

	      if (strcmp(rule_name, name) == 0)
		{
		  result = TRUE;
		  break;
		}
	    }
	}

      if (result)
	ochusha_combo_action_set_active_iter(OCHUSHA_COMBO_ACTION(threadlist_rule),
					     &iter);
    }
}


static void
redraw_threadlist(GtkWidget *view, BulletinBoard *board, void *data)
{
  if (board->get_rule() == NULL)
    {
      Controller *controller = static_cast<Controller *>(data);
      controller->refresh(board, false);
    }
}


void
BoardUI::set_default_threadlist_rule(const char *name)
{
  const OchushaProcedure &procedure
    = controller.get_scheme_manager().get_ochusha_procedure();
  const ThreadlistRuleInfo *info = procedure.lookup_threadlist_rule_info(name);
  assert(info != NULL);

  const char *previous_rule
    = controller.get_prefs().get_default_threadlist_rule();
  bool rule_changed = previous_rule == NULL || strcmp(previous_rule, name) != 0;
  if (rule_changed)
    {
      GtkTreeIter iter;
      if (gtk_tree_model_get_iter_first(threadlist_rule_model, &iter))
	{
	  gchar *label = NULL;
	  gtk_tree_model_get(threadlist_rule_model, &iter,
			     THREADLIST_RULE_LABEL, &label, -1);
	  g_free(label);
	  label = g_strdup_printf("%s (*)", info->get_label());
	  gtk_list_store_set(GTK_LIST_STORE(threadlist_rule_model), &iter,
			     THREADLIST_RULE_LABEL, label,
			     THREADLIST_RULE_NAME, NULL,
			     -1);
	}

      controller.get_prefs().set_default_threadlist_rule(name);
      controller.get_view_manager().foreach_board(redraw_threadlist,
						  &controller);
      BulletinBoard *board = controller.get_view_manager().get_current_board();
      if (board == NULL || board->get_rule() == NULL)
	set_current_threadlist_rule(NULL);
    }
}


void
BoardUI::update_threadlist_rules()
{
  const OchushaProcedure &procedure
    = controller.get_scheme_manager().get_ochusha_procedure();

  vector<ThreadlistRuleInfo *>::const_iterator iter
    = procedure.threadlist_rule_list_begin();
  vector<ThreadlistRuleInfo *>::const_iterator end_iter
    = procedure.threadlist_rule_list_end();

  const char *default_rule_name
    = controller.get_prefs().get_default_threadlist_rule();
  assert(default_rule_name != NULL);


  GtkTreeIter dummy_iter;
  GtkListStore *current_store = GTK_LIST_STORE(threadlist_rule_model);
  gtk_list_store_insert_with_values(current_store, &dummy_iter, G_MAXINT32,
				    THREADLIST_RULE_LABEL, g_strdup("dummy"),
				    THREADLIST_RULE_NAME, NULL,
				    -1);

  GtkListStore *default_store = GTK_LIST_STORE(default_threadlist_rule_model);
  ThreadlistRuleInfo *default_info = NULL;
  while (iter != end_iter)
    {
      ThreadlistRuleInfo *info = *iter++;

      gtk_list_store_insert_with_values(current_store, &dummy_iter, G_MAXINT32,
					THREADLIST_RULE_LABEL, info->get_label(),
					THREADLIST_RULE_NAME, info->get_name(),
					-1);

      GtkTreeIter tree_iter;
      gtk_list_store_insert_with_values(default_store, &tree_iter, G_MAXINT32,
					THREADLIST_RULE_LABEL, info->get_label(),
					THREADLIST_RULE_NAME, info->get_name(),
					-1);
      if (strcmp(info->get_name(), default_rule_name) == 0)
	{
	  default_info = info;
	  ochusha_combo_action_set_active_iter(OCHUSHA_COMBO_ACTION(default_threadlist_rule),
					       &tree_iter);
	}
    }

  if (default_info != NULL)
    {
      GtkTreeIter tree_iter;
      if (gtk_tree_model_get_iter_first(threadlist_rule_model, &tree_iter))
	{
	  gchar *label = NULL;
	  gtk_tree_model_get(threadlist_rule_model, &tree_iter,
			     THREADLIST_RULE_LABEL, &label, -1);
	  if (label != NULL)
	    g_free(label);

	  label = g_strdup_printf("%s (*)", default_info->get_label());
	  gtk_list_store_set(current_store, &tree_iter,
			     THREADLIST_RULE_LABEL, label,
			     THREADLIST_RULE_NAME, NULL,
			     -1);
	}
    }
}


static void
redraw_threadlist_all(GtkWidget *view, BulletinBoard *board, void *data)
{
  static_cast<Controller *>(data)->refresh(board, false);
}


void
BoardUI::show_hidden_threads_toggled_cb(GtkAction *action, BoardUI *ui)
{
  ui->controller.get_view_manager().foreach_board(redraw_threadlist_all,
						  &ui->controller);
}

}

