/* -*- c++ -*-
 * Copyright (c) 2003-2007 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: boardlist_ui.cc,v 1.30 2007/01/09 12:14:47 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "boardlist_ui.h"

#include "libochusha/bbs_table.h"
#include "libochusha/ref_count.h"

#include "command_pipe.h"
#include "controller.h"
#include "session_client.h"
#include "tree_view.h"
#include "ui_constants.h"

#include "libochushagtk/stock.h"

#include <cassert>
#include <list>

#include <gtk/gtk.h>


namespace Ochusha
{


BoardlistUI::BoardlistUI(Controller &controller)
  : TreeUIImpl(controller, &controller.get_bbs_table()),
    action_group(create_action_group()),
    table(dynamic_cast<BBSTable *>(get_root_folder())),
    periodic_save_id(0),
    save_on_idle_id(0)
{
  assert(table != NULL);

  gdk_threads_leave();

  bool result = table->read_boardlist_xml(controller.get_repository());

  gdk_threads_enter();

  g_signal_connect(controller.get_prefs().show_hidden_boardlist_items,
		   "toggled",
		   reinterpret_cast<GCallback>(show_hidden_boardlist_items_toggled_cb),
		   this);

  if (!result)
    controller.get_command_pipe().send("(refresh-boardlist)\n");

  periodic_save_id
    = g_timeout_add(PERIODIC_SAVE_TIMEOUT,
		    reinterpret_cast<GSourceFunc>(periodic_save_timeout_cb),
		    this);
}


BoardlistUI::~BoardlistUI()
{
  if (periodic_save_id != 0)
    g_source_remove(periodic_save_id);

  if (save_on_idle_id != 0)
    g_source_remove(save_on_idle_id);

  g_object_unref(action_group);
}


gboolean
BoardlistUI::periodic_save_timeout_cb(BoardlistUI *ui)
{
  gdk_threads_enter();

  vector<smart_ptr<TreeElement> >::iterator iter
    = ui->table->get_all_boards_begin();
  vector<smart_ptr<TreeElement> >::iterator end_iter
    = ui->table->get_all_boards_end();

  // ʲΥɤĿn˴ؤO(n^2)ʤΤ¸ԤꥹȤ;ˤĹ
  // ʤȤޤΤȤ곰
  while (iter != end_iter)
    {
      TreeElement *elm = *iter;
      BulletinBoard *board = dynamic_cast<BulletinBoard *>(elm);
      assert(board != NULL);
      int how_dirty = board->how_dirty();
      if (how_dirty == URGENT_SAVE_RECOMMENDED)
	{
	  list<DirtyBoard>::iterator res_iter = ui->urgent_list.begin();
	  list<DirtyBoard>::iterator res_end_iter = ui->urgent_list.end();
	  while (res_iter != res_end_iter)
	    {
	      if ((*res_iter).board == board)
		break;
	      res_iter++;
	    }
	  if (res_iter == res_end_iter)
	    ui->urgent_list.push_back(DirtyBoard(board));
	}
      else if (how_dirty == MINOR_MODIFICATION)
	{
	  list<DirtyBoard>::iterator res_iter = ui->non_urgent_list.begin();
	  list<DirtyBoard>::iterator res_end_iter = ui->non_urgent_list.end();
	  while (res_iter != res_end_iter)
	    {
	      if ((*res_iter).board == board)
		break;
	      res_iter++;
	    }
	  if (res_iter == res_end_iter)
	    ui->non_urgent_list.push_back(DirtyBoard(board));
	}
      else
	assert(how_dirty == NOT_DIRTY);

      iter++;
    }

  if (ui->save_on_idle_id == 0
      && (ui->urgent_list.begin() != ui->urgent_list.end()
	  || ui->non_urgent_list.begin() != ui->non_urgent_list.end()))
    {
      ui->save_on_idle_id
	= g_idle_add_full(G_PRIORITY_LOW,
			  reinterpret_cast<GSourceFunc>(save_on_idle_cb),
			  ui, NULL);
    }

  gdk_threads_leave();

  return TRUE;	// Ǹޤǽʤ
}


void
BoardlistUI::save_threadlist(BulletinBoard *board)
{
  // gdk_threads_enter()Ƥ
  gchar message[256];
  snprintf(message, 256, _("Saving threadlist of %s board"), board->get_name());
  guint message_id = gtk_statusbar_push(controller.get_statusbar(),
					controller.get_boardlist_context_id(),
					message);
  board->write_threadlist_xml(controller.get_repository());
  gtk_statusbar_remove(controller.get_statusbar(),
		       controller.get_boardlist_context_id(), message_id);
}


gboolean
BoardlistUI::save_on_idle_cb(BoardlistUI *ui)
{
#if 0
  fprintf(stderr, "BoardlistUI::save_on_idle_cb()\n");
#endif
  Lock *global_lock = Lock::get_global_lock();
  if (global_lock->trylock())
    {
      gdk_threads_enter();

      bool save_performed = false;
      while (ui->urgent_list.begin() != ui->urgent_list.end())
	{
	  DirtyBoard entry = ui->urgent_list.front();
	  ui->urgent_list.pop_front();
	  BulletinBoard *board = entry.board;
	  if (board->how_dirty() == URGENT_SAVE_RECOMMENDED)
	    {
	      save_performed = true;
	      ui->save_threadlist(board);
	      break;
	    }
	}

      list<DirtyBoard>::iterator iter = ui->non_urgent_list.begin();
      list<DirtyBoard>::iterator end_iter = ui->non_urgent_list.end();
      while (!save_performed && iter != end_iter)
	{
	  BulletinBoard *board = (*iter).board;
	  switch (board->how_dirty())
	    {
	    case NOT_DIRTY:
	      ui->non_urgent_list.erase(iter++);
	      break;

	    case URGENT_SAVE_RECOMMENDED:
	      save_performed = true;
	      ui->save_threadlist(board);
	      ui->non_urgent_list.erase(iter++);
	      break;

	    case MINOR_MODIFICATION:
	      if ((*iter).board->get_dirtyness() != (*iter).dirtyness)
		{
		  (*iter).dirtyness = (*iter).board->get_dirtyness();
		  iter++;
		  break;	// ĤϺϸ
		}
	      save_performed = true;
	      ui->save_threadlist(board);
	      ui->non_urgent_list.erase(iter++);
	      break;
	    }
	}

      gdk_threads_leave();

      global_lock->unlock();
    }

  gdk_threads_enter();

  bool result = !ui->urgent_list.empty() || !ui->non_urgent_list.empty();
  if (!result)
    ui->save_on_idle_id = 0;
#if 0
  fprintf(stderr, "save_on_idle_cb: result=%d\n", result);
#endif

  gdk_threads_leave();

  return result;
}


void
BoardlistUI::save_boardlist()
{
  assert(table != NULL);
  table->write_boardlist_xml(controller.get_repository());
}


GtkActionGroup *
BoardlistUI::create_action_group()
{
  static GtkActionEntry entries[] = {
    {
      "ClearCookies", GTK_STOCK_DELETE,
      N_("Clear Cookies"), "",
      NULL,
      reinterpret_cast<GCallback>(clear_cookies_cb)
    },
    {
      "SearchBoardlist", GTK_STOCK_FIND,
      N_("Search Boar_dlist"), "",
      N_("Search Boardlist"),
      reinterpret_cast<GCallback>(search_boardlist_cb)
    },
    {
      "RefreshBoardlist", GTK_STOCK_REFRESH,
      N_("Refresh Boar_dlist"), "",
      N_("Refresh Boardlist"),
      reinterpret_cast<GCallback>(refresh_boardlist_cb)
    },
    {
      "AddBoard", OCHUSHA_STOCK_ADD_BOARD,
      N_("_Add Board"), "",
      N_("Add Board"),
      reinterpret_cast<GCallback>(add_board_cb)
    },
    {
      "ManageBoardlist", NULL,
      N_("_Manage Boardlist"), NULL,
      N_("Manage Boardlist"),
      reinterpret_cast<GCallback>(manage_boardlist_cb)
    }
  };

  action_group = gtk_action_group_new("BoardlistActions");
  gtk_action_group_set_translation_domain(action_group, PACKAGE_NAME);
  gtk_action_group_add_actions(action_group, entries,
			       G_N_ELEMENTS(entries), this);

  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     action_group, 0);
  return action_group;
}


void
BoardlistUI::refresh_boardlist()
{
  assert(table != NULL);

  gdk_threads_leave();

  table->update_boardlist(controller.get_config(), controller.get_thread_pool(),
			  controller.get_broker());

  gdk_threads_enter();
}


void
BoardlistUI::clear_cookies_cb(GtkAction *action, BoardlistUI *ui)
{
  ui->table->clear_all_cookies();
}


void
BoardlistUI::search_boardlist_cb(GtkAction *action, BoardlistUI *ui)
{
  ui->master_view->setup_search(_("Find in Boardlist"));
  ui->controller.get_prefs().set_show_boardlist_sidebar(false);
  ui->controller.get_prefs().set_show_boardlist_sidebar(true);
}


void
BoardlistUI::refresh_boardlist_cb(GtkAction *action, BoardlistUI *ui)
{
  ui->refresh_boardlist();
}


class AddBoardHelper
{
private:
  BoardlistUI &ui;

  GtkEntry *name_entry;
  GtkEntry *url_entry;


  ~AddBoardHelper();

  static void destroy_cb(GtkWidget *widget, AddBoardHelper *helper);
  static void response_cb(GtkWidget *widget, gint response_id,
			  AddBoardHelper *helper);


public:
  AddBoardHelper(BoardlistUI &ui);
};


AddBoardHelper::AddBoardHelper(BoardlistUI &ui)
  : ui(ui)
{
  GtkWidget *dialog
    = gtk_dialog_new_with_buttons(_("Add Board"),
				  ui.controller.get_toplevel(),
				  GTK_DIALOG_DESTROY_WITH_PARENT,
				  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				  GTK_STOCK_OK, GTK_RESPONSE_OK,
				  NULL);
  gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);

  g_signal_connect(dialog, "response",
		   reinterpret_cast<GCallback>(response_cb), this);
  g_signal_connect(dialog, "destroy",
		   reinterpret_cast<GCallback>(destroy_cb), this);

  GtkWidget *outer_hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(outer_hbox);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
		     outer_hbox, TRUE, TRUE, 0);

  GtkWidget *icon = gtk_image_new_from_stock(GTK_STOCK_DIALOG_QUESTION,
					     GTK_ICON_SIZE_DIALOG);
  gtk_widget_show(icon);
  gtk_box_pack_start(GTK_BOX(outer_hbox), icon, FALSE, FALSE, 5);

  GtkWidget *widget = gtk_table_new(2, 2, FALSE);
  gtk_widget_show(widget);
  gtk_box_pack_start(GTK_BOX(outer_hbox), widget, TRUE, TRUE, 0);

  GtkTable *table = GTK_TABLE(widget);



  GtkWidget *label = gtk_label_new_with_mnemonic(_("_Name:"));
  gtk_widget_show(label);
  gtk_table_attach(table, label, 0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);


  GtkWidget *entry = gtk_entry_new();
  gtk_widget_show(entry);
  gtk_table_attach_defaults(table, entry, 1, 2, 0, 1);
  gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);

  name_entry = GTK_ENTRY(entry);
  gtk_entry_set_text(name_entry, _("New Board"));



  label = gtk_label_new_with_mnemonic(_("_URL:"));
  gtk_widget_show(label);
  gtk_table_attach(table, label, 0, 1, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);


  entry = gtk_entry_new();
  gtk_widget_show(entry);
  gtk_table_attach_defaults(table, entry, 1, 2, 1, 2);
  gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);

  url_entry = GTK_ENTRY(entry);
  gtk_entry_set_width_chars(url_entry, 48);
  gtk_entry_set_activates_default(url_entry, TRUE);

  gtk_widget_show(dialog);
}


AddBoardHelper::~AddBoardHelper()
{
}


void
AddBoardHelper::destroy_cb(GtkWidget *widget, AddBoardHelper *helper)
{
  delete helper;
}


void
AddBoardHelper::response_cb(GtkWidget *widget, gint response_id,
			    AddBoardHelper *helper)
{
  if (response_id == GTK_RESPONSE_OK)
    {
      const char *url = gtk_entry_get_text(helper->url_entry);
      BulletinBoard *board
	= helper->ui.controller.get_bbs_table().guess_board(url);
      if (board == NULL)
	{
	  board = new BulletinBoard();
	  char *board_url = BBSTable::guess_board_url(url);
	  if (board_url != NULL)
	    url = board_url;

	  board->set_base_url(url);
	  board->set_bbs_type(BBSTable::guess_board_type(url));

	  if (board_url != NULL)
	    free(board_url);

	  board->set_name(gtk_entry_get_text(helper->name_entry));
	}
      helper->ui.controller.get_board_properties_ui().add_board(board);
    }

  gtk_widget_hide(widget);
  gtk_widget_unrealize(widget);
  gtk_widget_destroy(widget);
}


void
BoardlistUI::add_board_cb(GtkAction *action, BoardlistUI *ui)
{
  new AddBoardHelper(*ui);
}


void
BoardlistUI::manage_boardlist_cb(GtkAction *action, BoardlistUI *ui)
{
}


void
BoardlistUI::show_hidden_boardlist_items_toggled_cb(GtkAction *action,
						    BoardlistUI *ui)
{
  ui->reconstruct();
}


const gchar *
BoardlistUI::get_element_icon_stock_id(TreeElement *element, gboolean expanded)
{
  if (element->is_folder())
    return TreeUIImpl::get_element_icon_stock_id(element, expanded);

  BulletinBoard *board = dynamic_cast<BulletinBoard *>(element);
  g_return_val_if_fail(board != NULL, NULL);

  switch (board->get_bbs_type())
    {
    case OCHUSHA_BBS_TYPE_PSEUDO_FOLDER:
    case OCHUSHA_BBS_TYPE_PASSIVE_FOLDER:
      return OCHUSHA_STOCK_BOOKMARK_FOLDER;

    case OCHUSHA_BBS_TYPE_ACTIVE_FOLDER:
      return OCHUSHA_STOCK_ACTIVE_FOLDER;

    default:
      return OCHUSHA_STOCK_BOARD;
    }
}


bool
BoardlistUI::show_all() const
{
  return controller.get_prefs().get_show_hidden_boardlist_items();
}


void
BoardlistUI::reconstruct()
{
  assert(table != NULL);
  list<smart_ptr<TreeElement> > &elements = table->get_elements();

  list<smart_ptr<TreeElement> >::iterator iter = elements.begin();
  list<smart_ptr<TreeElement> >::iterator end_iter = elements.end();

  Folder *my_folder = NULL;
  while (iter != end_iter)
    {
      TreeElement *element = *iter;
      if (element->is_folder())
	{
	  Folder *folder = dynamic_cast<Folder *>(element);
	  assert(folder != NULL);
	  if (folder->get_folder_type() == FOLDER_TYPE_OCHUSHA_MY_FOLDER)
	    {
	      my_folder = folder;
	      break;
	    }
	}
      iter++;
    }

  if (my_folder == NULL)
    {
      my_folder = table->RootFolder::folder_new();
      my_folder->set_folder_type(FOLDER_TYPE_OCHUSHA_MY_FOLDER);
      my_folder->set_name(_("My Folder"));
      table->prepend(my_folder);
    }

  TreeUIImpl::reconstruct();

  table->write_boardlist_xml(controller.get_repository());
}


const char *
BoardlistUI::get_popup_menu_root()
{
  return controller.get_prefs().get_show_hidden_boardlist_items()
    ? "/UglyBoardlistPopupMenu" : "/BoardlistPopupMenu";
}


const char *
BoardlistUI::get_folder_tree_popup_menu_root()
{
  return "/FolderTreePopupMenu";
}


GtkWidget *
BoardlistUI::get_menu_widget()
{
  GtkWidget *menu_widget
    = gtk_ui_manager_get_widget(controller.get_ui_manager(),
				"/MainMenu/BoardlistMenu");
  if (menu_widget == NULL)
    {
      fprintf(stderr, "Cannot find BoardlistMenu\n");
      return NULL;
    }

  return menu_widget;
}


void
BoardlistUI::open(TreeElement *element, bool in_tab, bool with_browser,
		  bool select_page)
{
  g_return_if_fail(!element->is_separator());

  if (!element->is_folder())
    {
      BulletinBoard *board = dynamic_cast<BulletinBoard *>(element);
      g_return_if_fail(board != NULL);

      controller.open(board, in_tab, with_browser, select_page);
      return;
    }

  if (!in_tab)
    return;

  Folder *folder = dynamic_cast<Folder *>(element);
  g_return_if_fail(folder != NULL);

  list<smart_ptr<TreeElement> >::const_iterator iter
    = folder->get_elements_begin();
  list<smart_ptr<TreeElement> >::const_iterator end_iter
    = folder->get_elements_end();

  while (iter != end_iter)
    {
      TreeElement *elm = *iter++;
      if (!elm->is_folder() && !elm->is_separator())
	{
	  BulletinBoard *board = dynamic_cast<BulletinBoard *>(elm);
	  if (board != NULL)
	    controller.open(board, true, false, select_page);
	}
    }
}


void
BoardlistUI::bookmark(TreeElement *element)
{
  controller.bookmark(dynamic_cast<BulletinBoard *>(element));
}


const char *
BoardlistUI::get_url(TreeElement *element)
{
  if (!element->is_folder())
    {
      BulletinBoard *board = dynamic_cast<BulletinBoard *>(element);
      if (board != NULL)
	return board->get_base_url();
    }
  return "";	// ޤ
}


const char *
BoardlistUI::get_ui_name()
{
  return "BoardlistView";
}


void
BoardlistUI::process_response(GtkDialog *dialog, int response_id,
			      TreeElement *element)
{
  if (element->is_folder())
    return;

  BulletinBoard *board = dynamic_cast<BulletinBoard *>(element);
  g_return_if_fail(board != NULL);

  controller.get_board_properties_ui().process_response(dialog, response_id,
							board);
}


bool
BoardlistUI::construct_properties_contents(GtkDialog *dialog,
					   TreeElement *element)
{
  if (element->is_folder())
    return false;

  BulletinBoard *board = dynamic_cast<BulletinBoard *>(element);
  g_return_val_if_fail(board != NULL, false);

  controller.get_board_properties_ui().construct_properties_contents(dialog,
								     board);
  return true;
}


void
BoardlistUI::open_properties(TreeElement *element)
{
  const char *name = element->get_name();
  if (name == NULL)
    name = _("Anonymous Item");

  char *title;
  if (element->is_folder())
    title = g_strdup_printf(_("Boardlist Folder's Properties for \"%s\""),
			    name);
  else
    title = g_strdup_printf(_("Properties for \"%s\" board"), name);

  // ʤǸʤ
  TreeUIImpl::open_properties(title, element);

  g_free(title);
}


bool
BoardlistUI::process_drop_data(Folder *src_folder, TreeElement *src_element,
			       Folder *drop_folder, TreeElement *drop_element,
			       GtkTreeViewDropPosition position, bool move)
{
  if (src_folder == drop_folder && src_element == drop_element)
    return false;	// ͭ͡ġ

  // src_elementİΤΤǤʤURLdropΤƱ񤤤Ȥ롣
  int folder_type = src_folder->get_folder_type();
  if ((folder_type == FOLDER_TYPE_ROOT_FOLDER
       && src_folder != get_root_folder())
      || folder_type == FOLDER_TYPE_BOOKMARKS_FOLDER)
    return process_drop_data(get_url(src_element), drop_folder, drop_element,
			     position);

  // ߤμǤBulletinBoardTreeElementǤꡢޤBulletinBoard
  // ĤФsingletonǤ뤳ȤݾڤƤ뤻ǡƱFolder
  // ƱBulletinBoardʣ뤳ȤϤǤʤξϰư롣
  Folder *dest_folder;
  if (position == GTK_TREE_VIEW_DROP_BEFORE
      || position == GTK_TREE_VIEW_DROP_AFTER
      || !drop_element->is_folder())
    dest_folder = drop_folder;
  else
    dest_folder = dynamic_cast<Folder *>(drop_element);
  g_return_val_if_fail(dest_folder != NULL, false);

  if (src_folder == dest_folder)
    move = true;

  if (move)
    return move_element(src_folder, src_element, drop_folder, drop_element,
			position);

  // 褺src_element->is_folder()ʾ̵ȿȤ롣
  if (src_element->is_folder())
    return false;

  smart_ptr<BulletinBoard> board = dynamic_cast<BulletinBoard *>(src_element);
  g_return_val_if_fail(board != NULL, false);

  new_element(dest_folder, board);
  return true;
}


bool
BoardlistUI::process_drop_data(const char *uri, Folder *drop_folder,
			       TreeElement *drop_element,
			       GtkTreeViewDropPosition position)
{
  return false;
}


}
