/* -*- c++ -*-
 * Copyright (c) 2003-2007 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: link_ui.cc,v 1.2 2007/01/01 03:18:25 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "link_ui.h"


#include "controller.h"

#include "libochushagtk/misc_utils.h"

#include <string>


namespace Ochusha
{


LinkUI::LinkUI(Controller &controller)
  : controller(controller),
    action_group(create_action_group())
{
}


LinkUI::~LinkUI()
{
  g_object_unref(action_group);
}


GtkActionGroup *
LinkUI::create_action_group()
{
  static GtkActionEntry entries[] = {
    {
      "OpenLinkLocation", GTK_STOCK_OPEN,
      N_("_Open"), "",
      N_("Open"),
      reinterpret_cast<GCallback>(open_link_location_cb)
    },
    {
      "OpenLinkLocationInTab", GTK_STOCK_OPEN,
      N_("Open in _Tab"), "",
      N_("Open in Tab"),
      reinterpret_cast<GCallback>(open_link_location_in_tab_cb)
    },
    {
      "OpenLinkLocationWithWebBrowser", GTK_STOCK_EXECUTE,
      N_("Open with _Web Browser"), NULL,
      N_("Open with Web Browser"),
      reinterpret_cast<GCallback>(open_link_location_with_web_browser_cb)
    },
    {
      "BookmarkLinkLocation", NULL,
      N_("_Bookmark"), NULL,
      N_("Bookmark"),
      reinterpret_cast<GCallback>(bookmark_link_location_cb)
    },
    {
      "CopyLinkLocation", GTK_STOCK_COPY,
      N_("_Copy URL"), "",
      N_("Copy URL"),
      reinterpret_cast<GCallback>(copy_link_location_cb)
    }
  };


  GtkActionGroup *action_group = gtk_action_group_new("LinkUIActions");

  gtk_action_group_set_translation_domain(action_group, PACKAGE_NAME);
  gtk_action_group_add_actions(action_group, entries,
			       G_N_ELEMENTS(entries), this);

  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     action_group, 0);
  
  g_signal_connect(controller.get_internal_clipboard(), "owner-change",
		   reinterpret_cast<GCallback>(owner_change_cb), this);

  return action_group;
}


void
LinkUI::owner_change_cb(GtkClipboard *clipboard, GdkEvent *event,
			LinkUI *ui)
{
  gboolean uri_available = ochusha_clipboard_wait_is_uri_available(clipboard);
  gtk_action_group_set_sensitive(ui->action_group, uri_available);
}


void
LinkUI::open_link_location_cb(GtkAction *action, LinkUI *ui)
{
  GtkClipboard *clipboard = ui->controller.get_internal_clipboard();
  gboolean uri_available = ochusha_clipboard_wait_is_uri_available(clipboard);
  if (!uri_available)
    return;

  gchar **uri_list = ochusha_clipboard_wait_for_uri(clipboard);
  g_return_if_fail(uri_list != NULL);

  gchar **uri_pos = uri_list;
  bool in_tab = false;
  while (*uri_pos != NULL)
    {
      const char *link = *uri_pos++;
      in_tab = in_tab || *uri_pos != NULL;
      ui->controller.open(link, in_tab, false, true);
    }

  g_strfreev(uri_list);
}


void
LinkUI::open_link_location_in_tab_cb(GtkAction *action, LinkUI *ui)
{
  GtkClipboard *clipboard = ui->controller.get_internal_clipboard();
  gboolean uri_available = ochusha_clipboard_wait_is_uri_available(clipboard);
  if (!uri_available)
    return;

  gchar **uri_list = ochusha_clipboard_wait_for_uri(clipboard);
  g_return_if_fail(uri_list != NULL);

  gchar **uri_pos = uri_list;
  while (*uri_pos != NULL)
    ui->controller.open(*uri_pos++, true, false, false);

  g_strfreev(uri_list);
}


void
LinkUI::open_link_location_with_web_browser_cb(GtkAction *action, LinkUI *ui)
{
  GtkClipboard *clipboard = ui->controller.get_internal_clipboard();
  gboolean uri_available = ochusha_clipboard_wait_is_uri_available(clipboard);
  if (!uri_available)
    return;

  gchar **uri_list = ochusha_clipboard_wait_for_uri(clipboard);
  g_return_if_fail(uri_list != NULL);

  gchar **uri_pos = uri_list;
  while (*uri_pos != NULL)
    ui->controller.open(*uri_pos++, true, true, true);

  g_strfreev(uri_list);
}


void
LinkUI::copy_link_location_cb(GtkAction *action, LinkUI *ui)
{
  GtkClipboard *clipboard = ui->controller.get_internal_clipboard();
  gboolean uri_available = ochusha_clipboard_wait_is_uri_available(clipboard);
  if (!uri_available)
    return;

  gchar **uri_list = ochusha_clipboard_wait_for_uri(clipboard);
  g_return_if_fail(uri_list != NULL);

  string buf;
  gchar **uri_pos = uri_list;
  while (*uri_pos != NULL)
    {
      if (buf.size() > 0)
	buf.append("\n");
      buf.append(*uri_pos++);
    }

  ui->controller.copy(buf.c_str());
  g_strfreev(uri_list);
}


void
LinkUI::bookmark_link_location_cb(GtkAction *action, LinkUI *ui)
{
  GtkClipboard *clipboard = ui->controller.get_internal_clipboard();
  gboolean uri_available = ochusha_clipboard_wait_is_uri_available(clipboard);
  if (!uri_available)
    return;

  gchar **uri_list = ochusha_clipboard_wait_for_uri(clipboard);
  g_return_if_fail(uri_list != NULL);

  gchar **uri_pos = uri_list;
  while (*uri_pos != NULL)
    ui->controller.bookmark(*uri_pos++, NULL);

  g_strfreev(uri_list);
}


}
