/* -*- c++ -*-
 * Copyright (c) 2003-2007 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: post_ui.cc,v 1.23 2007/01/10 10:14:37 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "post_ui.h"

#include "libochusha/misc_utils.h"
#include "libochusha/repository.h"

#include "controller.h"
#include "html_renderer.h"

#include "libochushagtk/hypertext_buffer.h"
#include "libochushagtk/hypertext_view.h"
#include "libochushagtk/response_editor.h"

#include <gtk/gtk.h>

#include <cassert>
#include <cctype>
#include <cstdio>
#include <cstdlib>
#include <cstring>


namespace Ochusha
{

#define OCHUSHA_AALIST_TXT	"aalist.txt"


class PostResultWindow;

class PostResultRenderer : public HTMLRenderer
{
private:
  PostResultWindow &window;

  guint in_form_element;


protected:
  virtual void start_element(const char *name, const char *const attrs[]);
  virtual void end_element(const char *name);


public:
  PostResultRenderer(Controller &controller,
		     PostResultWindow &window,
		     OchushaHypertextBuffer *buffer,
		     OchushaHypertextView *view);
};


class PostResultWindow
{
private:
  smart_ptr<PostResponseAgent> agent;

  OchushaHypertextBuffer *buffer;
  GtkWidget *widget;
  OchushaHypertextView *view;

  PostResultRenderer renderer;

  GtkWidget *window;
  GtkWidget *submit_button;

  time_t utc_time;
  char *submit;


  static void response_cb(GtkWidget *dialog, int response_id,
			  PostResultWindow *window);


public:
  PostResultWindow(Controller &controller, GtkWindow *parent,
		    PostResponseAgent *agent, const gchar *message);
  ~PostResultWindow();

  void set_time(const char *value);
  void set_submit(const char *value);

  GtkWidget *get_widget() { return window; }
};


PostResultRenderer::PostResultRenderer(Controller &controller,
				       PostResultWindow &window,
				       OchushaHypertextBuffer *buffer,
				       OchushaHypertextView *view)
  : HTMLRenderer(controller, buffer, view),
    window(window),
    in_form_element(0)
{
}


void
PostResultRenderer::start_element(const char *name, const char *const attrs[])
{
  if (strcasecmp(name, "form") == 0)
    {
      in_form_element = 1;
      return;
    }

  if (in_form_element)
    {
      if (strcasecmp(name, "input") == 0)
	{
	  const char *attr_name = NULL;
	  const char *attr_value = NULL;
	  int i = 0;
	  while (attrs[i * 2] != NULL)
	    {
	      if (strcasecmp(attrs[i * 2], "name") == 0)
		attr_name = attrs[i * 2 + 1];
	      else if (strcasecmp(attrs[i * 2], "value") == 0)
		attr_value = attrs[i * 2 + 1];
	      i++;
	    }
	  if (attr_name != NULL && attr_value != NULL)
	    {
	      if (strcasecmp(attr_name, "time") == 0)
		window.set_time(attr_value);
	      else if (strcasecmp(attr_name, "submit") == 0)
		window.set_submit(attr_value);
	    }
	}
      return;
    }
  HTMLRenderer::start_element(name, attrs);
}


void
PostResultRenderer::end_element(const char *name)
{
  if (strcasecmp(name, "form") == 0)
    {
      in_form_element = 0;
      return;
    }

  if (!in_form_element)
    HTMLRenderer::end_element(name);
}


PostResultWindow::PostResultWindow(Controller &controller,
				   GtkWindow *parent,
				   PostResponseAgent *agent,
				   const gchar *message)
  : agent(agent),
    buffer(ochusha_hypertext_buffer_new(NULL)),
    widget(ochusha_hypertext_view_new(buffer)),
    view(OCHUSHA_HYPERTEXT_VIEW(widget)),
    renderer(controller, *this, buffer, view),
    window(gtk_dialog_new_with_buttons(_("Post Result"),
				       parent,
				       GTK_DIALOG_DESTROY_WITH_PARENT,
				       GTK_STOCK_GO_BACK, GTK_RESPONSE_CANCEL,
				       NULL)),
    submit_button(NULL),
    utc_time(-1),
    submit(NULL)
{
  g_signal_connect(window, "response",
		   reinterpret_cast<GCallback>(response_cb), this);
  gtk_box_set_homogeneous(GTK_BOX(GTK_DIALOG(window)->action_area), FALSE);

  GtkWidget *scrolled = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_widget_show(widget);
  gtk_container_add(GTK_CONTAINER(scrolled), widget);
  gtk_widget_show(scrolled);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox),
		     scrolled, TRUE, TRUE, 0);
  renderer.render_html(message);

  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size(GTK_WINDOW(window), 600, 400);
}


PostResultWindow::~PostResultWindow()
{
  if (submit != NULL)
    free(submit);
}


void
PostResultWindow::response_cb(GtkWidget *dialog, int response_id,
			      PostResultWindow *window)
{
  if (response_id == GTK_RESPONSE_OK)
    {
      gdk_threads_leave();

      window->agent->retry_post(window->submit, window->utc_time);

      gdk_threads_enter();
    }

  gtk_widget_destroy(dialog);

  gdk_threads_leave();

  LockBlock lock_obj(Lock::get_global_lock());

  delete window;

  gdk_threads_enter();
}


void
PostResultWindow::set_time(const char *value)
{
  long t;
  if (sscanf(value, "%ld", &t) == 1)
    utc_time = t;
  else
    utc_time = -1;
}


void
PostResultWindow::set_submit(const char *value)
{
  assert(value != NULL);
  if (submit != NULL)
    free(submit);

  submit = strdup(value);
  if (submit_button == NULL)
    submit_button = gtk_dialog_add_button(GTK_DIALOG(window),
					  value, GTK_RESPONSE_OK);
  else
    gtk_button_set_label(GTK_BUTTON(submit_button), value);
}


ResponseEditor::ResponseEditor(Controller &controller, BBSThread *thread)
  : controller(controller),
    post_action(gtk_action_new("ResponseEditor::post",
			       _("_Post"),
			       _("Post This Response"),
			       GTK_STOCK_OK)),
    clear_action(gtk_action_new("ResponseEditor::clear",
				_("_Delete"),
				_("Delete Response"),
				GTK_STOCK_DELETE)),
    sage_button(gtk_check_button_new_with_label("sage")),
    be_button(gtk_toggle_button_new_with_label(_("Be"))),
    viewer_button(gtk_toggle_button_new_with_label(_("Viewer"))),
    editor(OCHUSHA_RESPONSE_EDITOR(ochusha_response_editor_new())),
    thread(thread),
    result_window(NULL),
    result_window_destroy_id(0),
    ensure_sage_on_idle_id(0)
{
  g_signal_connect(post_action, "activate",
		   reinterpret_cast<GCallback>(post_activate_cb), this);
  g_signal_connect(clear_action, "activate",
		   reinterpret_cast<GCallback>(clear_activate_cb), this);
  gtk_action_set_sensitive(post_action, FALSE);
  gtk_action_set_sensitive(clear_action, FALSE);

  g_signal_connect(editor->text_view, "populate_popup",
		   reinterpret_cast<GCallback>(populate_popup_cb), this);

  g_signal_connect(sage_button, "toggled",
		   reinterpret_cast<GCallback>(sage_button_toggled_cb), this);
  gtk_widget_show(sage_button);
  gtk_box_pack_start(editor->hbox, sage_button, FALSE, FALSE, 0);
  gtk_button_set_focus_on_click(GTK_BUTTON(sage_button), FALSE);

  g_signal_connect(be_button, "toggled",
		   reinterpret_cast<GCallback>(be_button_toggled_cb), this);
  gtk_box_pack_start(editor->hbox, be_button, FALSE, FALSE, 0);
  gtk_button_set_focus_on_click(GTK_BUTTON(be_button), FALSE);

  g_signal_connect(viewer_button, "toggled",
		   reinterpret_cast<GCallback>(viewer_button_toggled_cb), this);
  gtk_box_pack_start(editor->hbox, viewer_button, FALSE, FALSE, 0);
  gtk_button_set_focus_on_click(GTK_BUTTON(viewer_button), FALSE);

  reload_aa();

  g_signal_connect(editor->name_combo, "changed",
		   reinterpret_cast<GCallback>(name_changed_cb), this);
  g_signal_connect(editor->mail_combo, "changed",
		   reinterpret_cast<GCallback>(mail_changed_cb), this);

  GtkTextBuffer *buffer = gtk_text_view_get_buffer(editor->text_view);
  g_signal_connect(buffer, "changed",
		   reinterpret_cast<GCallback>(message_changed_cb), this);
  update_current_entries();
  update_actions(buffer);


  {
    list<char *>::const_iterator iter
      = controller.get_prefs().get_recent_name_list_begin();
    list<char *>::const_iterator end_iter
      = controller.get_prefs().get_recent_name_list_end();

    while (iter != end_iter)
      {
	const char *name = *iter++;
	assert(name != NULL);
	ochusha_response_editor_add_name(editor, name);
      }
  }

  {
    list<char *>::const_iterator iter
      = controller.get_prefs().get_recent_mail_list_begin();
    list<char *>::const_iterator end_iter
      = controller.get_prefs().get_recent_mail_list_end();

    while (iter != end_iter)
      {
	const char *mail = *iter++;
	assert(mail != NULL);
	ochusha_response_editor_add_mail(editor, mail);
      }
  }
}


ResponseEditor::~ResponseEditor()
{
  if (result_window_destroy_id != 0)
    g_signal_handler_disconnect(result_window, result_window_destroy_id);

  if (ensure_sage_on_idle_id != 0)
    g_source_remove(ensure_sage_on_idle_id);

  g_object_unref(post_action);
  g_object_unref(clear_action);
}


void
ResponseEditor::populate_popup_cb(GtkTextView *view, GtkMenu *menu,
				  ResponseEditor *editor)
{
  editor->controller.get_scheme_manager()
    .attach_external_text_tools_menu(view, menu);
}


void
ResponseEditor::update_actions(GtkTextBuffer *buffer)
{
  if (static_cast<BBSThread *>(thread) == NULL)
    return;

  int char_count = gtk_text_buffer_get_char_count(buffer);
  if (gtk_action_get_sensitive(post_action))
    {
      g_return_if_fail(thread->is_alive());
      if (char_count == 0)
	gtk_action_set_sensitive(post_action, FALSE);
    }
  else if (thread->is_alive() && char_count > 0)
    gtk_action_set_sensitive(post_action, TRUE);

  gtk_action_set_sensitive(clear_action, char_count > 0);
}


gboolean
ResponseEditor::ensure_sage_on_idle_cb(ResponseEditor *editor)
{
  gdk_threads_enter();

  editor->ensure_sage_on_idle_id = 0;

  const char *mail = ochusha_response_editor_get_mail(editor->editor);
  if (mail == NULL || *mail == '\0')
    {
      ochusha_response_editor_set_mail(editor->editor, "sage");

      gdk_threads_leave();

      return FALSE;
    }

  // ǼĤǻȤʸencodingѴƤå٤ġ
  if (strstr(mail, "sage") != NULL)
    {
      gdk_threads_leave();

      return FALSE;
    }

  // UTF-8256ХȤۤʬڤΤƤ뤬ʤġġ
  char mail_buf[256];
  snprintf(mail_buf, 256, "%s", mail);	// ݤʤΤǤǥԡ
  char *sharp_pos = strchr(mail_buf, '#');

  char sage_mail_buf[256];
  if (sharp_pos == NULL)
    snprintf(sage_mail_buf, 256, "%ssage", mail_buf);
  else
    {
      *sharp_pos = '\0';
      snprintf(sage_mail_buf, 256, "%ssage#%s", mail_buf, sharp_pos + 1);
    }
  ochusha_response_editor_set_mail(editor->editor, sage_mail_buf);

  gdk_threads_leave();

  return FALSE;
}


void
ResponseEditor::name_changed_cb(GtkComboBox *widget, ResponseEditor *editor)
{
}


void
ResponseEditor::mail_changed_cb(GtkComboBox *widget, ResponseEditor *editor)
{
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(editor->sage_button))
      && editor->ensure_sage_on_idle_id == 0)
    editor->ensure_sage_on_idle_id
      = g_idle_add(reinterpret_cast<GSourceFunc>(ensure_sage_on_idle_cb),
		   editor);
  return;
}


void
ResponseEditor::message_changed_cb(GtkTextBuffer *buffer,
				   ResponseEditor *editor)
{
  editor->update_actions(buffer);
}


void
ResponseEditor::post_activate_cb(GtkAction *action, ResponseEditor *editor)
{
#if 0
  fprintf(stderr, "ResponseEditor::post_activate_cb()\n");
#endif
  editor->post();
}


void
ResponseEditor::clear_activate_cb(GtkAction *action, ResponseEditor *editor)
{
#if 0
  fprintf(stderr, "ResponseEditor::clear_activate_cb()\n");
#endif
  editor->clear();
}


void
ResponseEditor::sage_button_toggled_cb(GtkToggleButton *button,
				       ResponseEditor *editor)
{
  if (gtk_toggle_button_get_active(button)
      && editor->ensure_sage_on_idle_id == 0)
    editor->ensure_sage_on_idle_id
      = g_idle_add(reinterpret_cast<GSourceFunc>(ensure_sage_on_idle_cb),
		   editor);

  // sageåȤsageä٤ġġ
}


void
ResponseEditor::be_button_toggled_cb(GtkToggleButton *button,
				     ResponseEditor *editor)
{
  if (gtk_toggle_button_get_active(button)
      && !editor->controller.get_bbs2ch().login_2ch_be())
    gtk_toggle_button_set_active(button, FALSE);
}


void
ResponseEditor::viewer_button_toggled_cb(GtkToggleButton *button,
					 ResponseEditor *editor)
{
  if (gtk_toggle_button_get_active(button)
      && !editor->controller.get_bbs2ch().login_2ch_viewer())
    gtk_toggle_button_set_active(button, FALSE);
}


void
ResponseEditor::update_current_entries()
{
  if (thread != NULL)
    {
      const char *last_name = thread->get_last_name();
      if (last_name != NULL)
	{
	  ochusha_response_editor_add_name(editor, last_name);
	  ochusha_response_editor_set_name(editor, last_name);
	}
      const char *name = thread->get_board()->get_last_name();
      if (name != NULL)
	{
	  ochusha_response_editor_add_name(editor, name);
	  if (last_name == NULL)
	    ochusha_response_editor_set_name(editor, name);
	}

      const char *last_mail = thread->get_last_mail();
      if (last_mail != NULL)
	{
	  ochusha_response_editor_add_mail(editor, last_mail);
	  ochusha_response_editor_set_mail(editor, last_mail);
	}
      const char *mail = thread->get_board()->get_last_mail();
      if (mail != NULL)
	{
	  ochusha_response_editor_add_mail(editor, mail);
	  if (last_mail == NULL)
	    ochusha_response_editor_set_mail(editor, mail);
	}
    }
}


void
ResponseEditor::remember_current_entries()
{
  g_return_if_fail(static_cast<BBSThread *>(thread) != NULL);

  const char *last_name = ochusha_response_editor_get_name(editor);
  thread->set_last_name(last_name);

  const char *tmp_pos = last_name;
  if (tmp_pos != NULL)
    {
      while (*tmp_pos != '\0')
	{
	  if (!isdigit(*tmp_pos))
	    break;
	  ++tmp_pos;
	}

      if (*tmp_pos != '\0' || tmp_pos == last_name)
	{
	  thread->get_board()->set_last_name(last_name);
	  controller.get_prefs().register_recent_name(last_name);
	}
    }

  const char *last_mail = ochusha_response_editor_get_mail(editor);
  thread->set_last_mail(last_mail);
  thread->get_board()->set_last_mail(last_mail);
  controller.get_prefs().register_recent_mail(last_mail);
}


void
ResponseEditor::set_thread(BBSThread *thread)
{
  if (this->thread == thread && agent != NULL)
    return;

  if (this->thread != NULL)
    {
      remember_current_entries();

      if (agent != NULL)
	{
	  gdk_threads_leave();

	  agent->post_finished.disconnect(this);
	  agent->post_failed.disconnect(this);
	  agent->check_cookie.disconnect(this);

	  gdk_threads_enter();
	}
    }

  this->thread = thread;
  update_current_entries();
  update_actions(gtk_text_view_get_buffer(editor->text_view));

  int bbs_type;
  if (thread != NULL)
    {
      bbs_type = thread->get_board()->get_bbs_type();

      gdk_threads_leave();

      agent = thread->get_post_agent(controller.get_config(),
				     controller.get_broker());
      if (agent != NULL)
	{
	  agent->post_finished.connect(this, &ResponseEditor::post_finished_cb);
	  agent->post_failed.connect(this, &ResponseEditor::post_failed_cb);
	  agent->check_cookie.connect(this, &ResponseEditor::check_cookie_cb);
	}

      gdk_threads_enter();
    }
  else
    {
      bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;

      gdk_threads_leave();

      agent = NULL;

      gdk_threads_enter();
    }

  /*
   * ǼĤμˤꡢɤ"sage"Ǥʤäꡢ⤽sage
   * ǰ¸ߤʤ礬롣ɤ󤫡ġġ
   */
  switch (bbs_type)
    {
    case OCHUSHA_BBS_TYPE_2CH:
    case OCHUSHA_BBS_TYPE_2CH_BE:
    case OCHUSHA_BBS_TYPE_BBSPINK:
      gtk_widget_show(be_button);
      gtk_widget_show(viewer_button);
      break;

    default:
      gtk_widget_hide(be_button);
      gtk_widget_hide(viewer_button);
      break;
    }
}


void
ResponseEditor::post_finished_cb(PostResponseAgent *agent)
{
  gdk_threads_enter();

  gtk_widget_set_sensitive(GTK_WIDGET(editor), TRUE);
  gtk_action_set_sensitive(post_action, TRUE);
  gtk_action_set_sensitive(clear_action, TRUE);

  clear();
  controller.refresh(agent->get_thread());

  notify_post_finished();

  gdk_threads_leave();
}


void
ResponseEditor::result_window_destroy_cb(GtkWidget *widget,
					 ResponseEditor *editor)
{
  g_return_if_fail(editor->result_window == widget);

  gtk_widget_set_sensitive(GTK_WIDGET(editor->editor), TRUE);
  gtk_action_set_sensitive(editor->post_action, TRUE);
  gtk_action_set_sensitive(editor->clear_action, TRUE);

  editor->result_window_destroy_id = 0;
  editor->result_window = NULL;
}


void
ResponseEditor::post_failed_cb(PostResponseAgent *agent, const char *reason)
{
#if 0
  fprintf(stderr, "ResponseEditor::post_failed_cb\n");
  if (reason != NULL)
    fprintf(stderr, "%s\n", reason);
  else
    fprintf(stderr, "Failed due to unknown reason\n");
#endif
  if (reason != NULL && strcasestr(reason, "<html>") != NULL)
    {
      gdk_threads_enter();

      if (result_window != NULL)
	g_signal_handler_disconnect(result_window, result_window_destroy_id);

      PostResultWindow *win = new PostResultWindow(controller,
						   controller.get_toplevel(),
						   agent, reason);
      result_window = win->get_widget();
      result_window_destroy_id
	= g_signal_connect(result_window, "destroy",
		reinterpret_cast<GCallback>(result_window_destroy_cb), this);
      gtk_widget_show_all(result_window);

      gdk_threads_leave();

      return;
    }
  else
    {
      gdk_threads_enter();

      gtk_widget_set_sensitive(GTK_WIDGET(editor), TRUE);
      gtk_action_set_sensitive(post_action, TRUE);
      gtk_action_set_sensitive(clear_action, TRUE);

      gdk_threads_leave();
    }
  if (reason == NULL)
    fprintf(stderr, "Post failed by unknown reason\n");
  else
    fprintf(stderr, "Post failed by: %s\n", reason);
}


void
ResponseEditor::check_cookie_cb(PostResponseAgent *agent, const char *message)
{
  if (message != NULL && strcasestr(message, "<html>") != NULL)
    {
      gdk_threads_enter();

      if (result_window != NULL)
	g_signal_handler_disconnect(result_window, result_window_destroy_id);

      PostResultWindow *win = new PostResultWindow(controller,
						   controller.get_toplevel(),
						   agent, message);
      result_window = win->get_widget();
      result_window_destroy_id
	= g_signal_connect(result_window, "destroy",
		reinterpret_cast<GCallback>(result_window_destroy_cb), this);
      gtk_widget_show_all(result_window);

      gdk_threads_leave();

      return;
    }
  gdk_threads_enter();

  gtk_widget_set_sensitive(GTK_WIDGET(editor), TRUE);
  gtk_action_set_sensitive(post_action, TRUE);
  gtk_action_set_sensitive(clear_action, TRUE);

  gdk_threads_leave();
}


void
ResponseEditor::notify_post_finished()
{
}


void
ResponseEditor::post()
{
  g_return_if_fail(agent != NULL);

  remember_current_entries();

  gchar *message = ochusha_response_editor_get_response(editor);
  g_return_if_fail(message != NULL);
  g_return_if_fail(*message != '\0');

  gtk_widget_set_sensitive(GTK_WIDGET(editor), FALSE);
  gtk_action_set_sensitive(post_action, FALSE);
  gtk_action_set_sensitive(clear_action, FALSE);

  switch (thread->get_board()->get_bbs_type())
    {
    case OCHUSHA_BBS_TYPE_2CH:
    case OCHUSHA_BBS_TYPE_2CH_BE:
    case OCHUSHA_BBS_TYPE_BBSPINK:
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(be_button)))
	agent->set_2ch_be_session_id(controller.get_bbs2ch()
				     .get_session_id_2ch_be());
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(viewer_button)))
	agent->set_2ch_viewer_session_id(controller.get_bbs2ch()
					 .get_session_id_2ch_viewer());
    }

  gdk_threads_leave();

  agent->try_post(ochusha_response_editor_get_name(editor),
		  ochusha_response_editor_get_mail(editor),
		  message);

  gdk_threads_enter();

  g_free(message);
}


void
ResponseEditor::clear()
{
  ochusha_response_editor_clear_response(editor);
}


void
ResponseEditor::reload_aa()
{
  ochusha_response_editor_reset_aa_list(editor);

  int fd = controller.get_repository().open_file(OCHUSHA_AALIST_TXT, O_RDONLY);
  if (fd < 0)
    {
      fd = open(PKGDATADIR "/" OCHUSHA_AALIST_TXT, O_RDONLY);
      if (fd < 0)
	return;
    }

  struct stat sb;
  if (fstat(fd, &sb) < 0)
    {
      close(fd);
      return;
    }

  char *buf = static_cast<char *>(malloc(sb.st_size + 1));
  ssize_t size = read(fd, buf, sb.st_size);

  char *cur_pos = buf;
  while (cur_pos - buf < size)
    {
      char *eol_pos = mempbrk(cur_pos, "\r\n", cur_pos - (buf + size));
      ssize_t len;
      if (eol_pos == NULL)
	eol_pos = buf + size;

      len = eol_pos - cur_pos;
      if (len > 1)
	{
	  *eol_pos = '\0';
	  ochusha_response_editor_add_aa(editor, cur_pos);
	}
      cur_pos = eol_pos + 1;
    }
  free(buf);
  close(fd);
}


GtkWidget *
ResponseEditor::get_widget()
{
  return GTK_WIDGET(editor);
}


EmbededResponseEditor::EmbededResponseEditor(Controller &controller)
  : ResponseEditor(controller, NULL),
    frame(gtk_frame_new(NULL)),
    hbox(gtk_hbox_new(FALSE, 6)),
    button_box(gtk_vbutton_box_new()),
    post_button(gtk_button_new_with_mnemonic(_("_Post"))),
    delete_button(gtk_button_new_with_mnemonic(_("_Delete")))
{
  GtkWidget *editor = ResponseEditor::get_widget();
  gtk_widget_show(editor);
  gtk_box_pack_start(GTK_BOX(hbox), editor, TRUE, TRUE, 0);

  gtk_button_set_image(GTK_BUTTON(post_button),
		       gtk_image_new_from_stock(GTK_STOCK_OK,
						GTK_ICON_SIZE_BUTTON));
  gtk_button_set_focus_on_click(GTK_BUTTON(post_button), FALSE);
  gtk_action_connect_proxy(post_action, post_button);
  gtk_widget_show(post_button);
  gtk_box_pack_start(GTK_BOX(button_box), post_button, FALSE, FALSE, 0);

  gtk_button_set_image(GTK_BUTTON(delete_button),
		       gtk_image_new_from_stock(GTK_STOCK_DELETE,
						GTK_ICON_SIZE_BUTTON));
  gtk_button_set_focus_on_click(GTK_BUTTON(delete_button), FALSE);
  gtk_action_connect_proxy(clear_action, delete_button);
  gtk_widget_show(delete_button);
  gtk_box_pack_start(GTK_BOX(button_box), delete_button, FALSE, FALSE, 0);

  gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box),
			    GTK_BUTTONBOX_SPREAD);
  gtk_widget_show(button_box);
  gtk_box_pack_end(GTK_BOX(hbox), button_box, FALSE, FALSE, 0);


  gtk_widget_show(hbox);
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 2);
  gtk_container_add(GTK_CONTAINER(frame), hbox);
}


EmbededResponseEditor::~EmbededResponseEditor()
{
  gtk_action_disconnect_proxy(post_action, post_button);
  gtk_action_disconnect_proxy(clear_action, delete_button);
}


void
EmbededResponseEditor::set_thread(BBSThread *thread)
{
  ResponseEditor::set_thread(thread);
}


GtkWidget *
EmbededResponseEditor::get_widget()
{
  return frame;
}


ResponseEditorWindow::ResponseEditorWindow(Controller &controller,
					   BBSThread *thread)
  : ResponseEditor(controller, NULL),
    window(gtk_dialog_new_with_buttons(_("Write Response"),
				       controller.get_toplevel(),
				       GTK_DIALOG_DESTROY_WITH_PARENT,
				       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				       GTK_STOCK_DELETE, GTK_RESPONSE_REJECT,
				       GTK_STOCK_OK, GTK_RESPONSE_OK,
				       NULL)),
    vbox(GTK_DIALOG(window)->vbox),
    title_hbox(gtk_hbox_new(FALSE, 5)),
    title_label(gtk_label_new(NULL)),
    board_label(gtk_label_new(NULL)),
    url_hbox(gtk_hbox_new(FALSE, 5)),
    url_entry(gtk_entry_new()),
    keep_button(gtk_check_button_new_with_label(_("Keep This Window After Posting")))
{
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size(GTK_WINDOW(window), -1, 250);

  g_signal_connect(window, "response",
		   reinterpret_cast<GCallback>(response_cb), this);


  gtk_label_set_justify(GTK_LABEL(title_label), GTK_JUSTIFY_RIGHT);
  gtk_widget_show(title_label);
  gtk_box_pack_start(GTK_BOX(title_hbox), title_label, TRUE, TRUE, 0);

  gtk_label_set_justify(GTK_LABEL(board_label), GTK_JUSTIFY_LEFT);
  gtk_widget_show(board_label);
  gtk_box_pack_end(GTK_BOX(title_hbox), board_label, FALSE, FALSE, 0);

  gtk_widget_show(title_hbox);
  gtk_box_pack_start(GTK_BOX(vbox), title_hbox, FALSE, FALSE, 0);


  GtkWidget *tmp_widget = gtk_label_new("URL: ");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(url_hbox), tmp_widget, FALSE, FALSE, 0);

  gtk_editable_set_editable(GTK_EDITABLE(url_entry), FALSE);
  GTK_WIDGET_UNSET_FLAGS(url_entry, GTK_CAN_FOCUS);
  gtk_widget_show(url_entry);
  gtk_box_pack_start(GTK_BOX(url_hbox), url_entry, TRUE, TRUE, 0);

  gtk_widget_show(url_hbox);
  gtk_box_pack_start(GTK_BOX(vbox), url_hbox, FALSE, FALSE, 0);


  tmp_widget = ResponseEditor::get_widget();
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, TRUE, TRUE, 0);


  gtk_widget_show(keep_button);
  gtk_box_pack_start(GTK_BOX(vbox), keep_button, FALSE, FALSE, 0);

  g_signal_connect(post_action, "notify::sensitive",
		   reinterpret_cast<GCallback>(notify_sensitive_cb), this);
  g_signal_connect(clear_action, "notify::sensitive",
		   reinterpret_cast<GCallback>(notify_sensitive_cb), this);
  g_object_notify(G_OBJECT(post_action), "sensitive");
  g_object_notify(G_OBJECT(clear_action), "sensitive");

  set_thread(thread);
}


ResponseEditorWindow::~ResponseEditorWindow()
{
#ifdef VERBOSE_DESTRUCTOR
  fprintf(stderr, "ResponseEditorWindow(@%p) destructed\n", this);
#endif
}


void
ResponseEditorWindow::notify_sensitive_cb(GtkAction *action, GParamSpec *arg1,
					  ResponseEditorWindow *window)
{
  gint response_id;
  if (action == window->post_action)
    response_id = GTK_RESPONSE_OK;
  else if (action == window->clear_action)
    response_id = GTK_RESPONSE_REJECT;
  else
    return;

  gtk_dialog_set_response_sensitive(GTK_DIALOG(window->window),
				    response_id,
				    gtk_action_get_sensitive(action));
}


void
ResponseEditorWindow::response_cb(GtkWidget *dialog, int response_id,
				  ResponseEditorWindow *window)
{
  switch (response_id)
    {
    case GTK_RESPONSE_OK:
      window->post();
      return;

    case GTK_RESPONSE_REJECT:
      window->clear();
      return;

    default:
      break;
    }

  gtk_widget_destroy(dialog);

  gdk_threads_leave();

  LockBlock lock_obj(Lock::get_global_lock());

  delete window;

  gdk_threads_enter();
}


void
ResponseEditorWindow::notify_post_finished()
{
  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(keep_button)))
    gtk_dialog_response(GTK_DIALOG(window), GTK_RESPONSE_CANCEL);
}


void
ResponseEditorWindow::set_thread(BBSThread *thread)
{
  if (thread != NULL)
    {
      gtk_label_set_text(GTK_LABEL(title_label), thread->get_title());

      char tmp_buf[256];
      snprintf(tmp_buf, 256, _("@ %s board"), thread->get_board()->get_name());
      gtk_label_set_text(GTK_LABEL(board_label), tmp_buf);

      char *url = thread->get_url();
      gtk_entry_set_text(GTK_ENTRY(url_entry), url);
      free(url);
    }
  else
    {
      gtk_label_set_text(GTK_LABEL(title_label), NULL);
      gtk_label_set_text(GTK_LABEL(board_label), NULL);

      gtk_entry_set_text(GTK_ENTRY(url_entry), "");
    }

  ResponseEditor::set_thread(thread);
}


GtkWidget *
ResponseEditorWindow::get_widget()
{
  return window;
}

}
