/* -*- c++ -*-
 * Copyright (c) 2003-2007 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: preferences_ui.cc,v 1.18 2007/01/04 06:40:30 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "preferences_ui.h"

#include "libochusha/configuration.h"
#include "controller.h"
#include "preferences.h"
#include "session_client.h"
#include "ui_constants.h"

#include <gtk/gtk.h>


namespace Ochusha
{

PreferencesUI::PreferencesUI(Controller &controller)
  : controller(controller),
    config(controller.get_config()),
    prefs(controller.get_prefs()),
    a_bone_settings(NULL),
    dialog(NULL),
    web_browser_template_entry(NULL),
    downloader_template_entry(NULL),
    popup_delay_spin_button(NULL),
    popdown_delay_spin_button(NULL),
    inline_image_width_spin_button(NULL),
    inline_image_height_spin_button(NULL),
    popup_image_width_spin_button(NULL),
    popup_image_height_spin_button(NULL),
    maximum_number_of_image_cache_files_spin_button(NULL),
    capacity_of_image_cache_spin_button(NULL),
    bookmark_text_buffer(NULL),
    focus_follow_mouse_button(NULL),
    default_open_in_tab_button(NULL),
    enable_popup_id_button(NULL),
    auto_refresh_go(NULL),
    auto_refresh_wait(NULL),
    state_color_settings_frame(NULL),
    ensure_5digits_only_on_idle_id(0)
{
}


PreferencesUI::~PreferencesUI()
{
  if (ensure_5digits_only_on_idle_id != 0)
    g_source_remove(ensure_5digits_only_on_idle_id);

  if (dialog != NULL)
    gtk_widget_destroy(dialog);

  if (a_bone_settings != NULL)
    delete a_bone_settings;
}


void
PreferencesUI::show_dialog()
{
  if (dialog == NULL)
    setup_dialog();

  gtk_widget_show(dialog);
  return;
}


GtkWidget *
PreferencesUI::create_general_settings_frame()
{
  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);



  // ֥֥饦
  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
  GtkWidget *tmp_widget = gtk_label_new(_("External Web Browser:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);

  tmp_widget = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(tmp_widget), prefs.get_web_browser_template());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 4);
  web_browser_template_entry = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(changed_cb), this);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 4);



  // 
  hbox = gtk_hbox_new(FALSE, 0);
  tmp_widget = gtk_label_new(_("External Downloader:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);

  tmp_widget = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(tmp_widget), prefs.get_downloader_template());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 4);
  downloader_template_entry = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(changed_cb), this);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 4);



  // 
  hbox = gtk_hbox_new(FALSE, 0);
  tmp_widget = gtk_label_new(_("(%s will be replaced with URL)"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_end(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);



  // ȥ
  tmp_widget = gtk_check_button_new_with_label(_("Use Internal Tiny Downloader"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_use_internal_downloader());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.use_internal_downloader),
			   tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Focus Follows Mouse"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_focus_follow_mouse());
  focus_follow_mouse_button = tmp_widget;
  gtk_widget_set_no_show_all(tmp_widget, TRUE);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.focus_follow_mouse), tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Open in Tab by Default"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_default_open_in_tab());
  default_open_in_tab_button = tmp_widget;
  gtk_widget_set_no_show_all(tmp_widget, TRUE);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.default_open_in_tab), tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Select the Tab When a New Tab is Opened"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_select_tab_when_opened());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.select_tab_when_opened),
			   tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Place Threadlist View Pane and Thread View Pane Vertically"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_vertical_placement());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.vertical_placement), tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Always Show Threadlist View Pane"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_board_pane_always_show());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.board_pane_always_show),
			   tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Always Show Board Tab"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_board_tab_always_show());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.board_tab_always_show),
			   tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Always Show Thread View Pane"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_thread_pane_always_show());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.thread_pane_always_show),
			   tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Always Show Thread Tab"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_thread_tab_always_show());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.thread_tab_always_show),
			   tmp_widget);


  GtkWidget *frame = gtk_frame_new(_("Settings of Ochusha"));
  gtk_widget_show(outer_vbox);
  gtk_container_add(GTK_CONTAINER(frame), outer_vbox);

  return frame;
}


GtkWidget *
PreferencesUI::create_list_view_settings_frame()
{
  GdkColor black = { 0, 0, 0, 0 };
  GdkColor white = { 0, 0xffff, 0xffff, 0xffff };
  GdkColor *fg = NULL;
  GdkColor *bg = NULL;

  GtkWidget *scrolled = controller.get_boardlist_ui().get_widget();
  if (scrolled != NULL && GTK_IS_SCROLLED_WINDOW(scrolled))
    {
      GtkWidget *tree_view = gtk_bin_get_child(GTK_BIN(scrolled));
      if (tree_view != NULL)
	{
	  GtkStyle *style = gtk_rc_get_style(tree_view);
	  fg = &style->text[GTK_STATE_NORMAL];
	  bg = &style->base[GTK_STATE_NORMAL];
	}
    }

  if (fg == NULL)
    fg = &black;

  if (bg == NULL)
    bg = &white;

  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);


  GtkWidget *hbox = gtk_hbutton_box_new();

  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
  GdkColor *color = prefs.get_fg_normal();
  gboolean color_set = TRUE;
  if (color == NULL)
    {
      color = fg;
      color_set = FALSE;
    }
  GtkWidget *tmp_widget = gtk_color_button_new_with_color(color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List Foreground Normal Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  fg_normal_color_button = tmp_widget;

  tmp_widget = gtk_check_button_new_with_label(_("Normal"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), color_set);
  g_signal_connect(tmp_widget, "toggled",
		   reinterpret_cast<GCallback>(color_set_toggled_cb), this);
  gtk_widget_show(tmp_widget);
  fg_normal_color_set_button = tmp_widget;
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  color = prefs.get_fg_emph();
  if (color == NULL)
    {
      color = fg;
      color_set = FALSE;
    }
  else
    color_set = TRUE;
  tmp_widget = gtk_color_button_new_with_color(color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List Foreground Emphasized Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  fg_emph_color_button = tmp_widget;

  tmp_widget = gtk_check_button_new_with_label(_("Emph"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), color_set);
  g_signal_connect(tmp_widget, "toggled",
		   reinterpret_cast<GCallback>(color_set_toggled_cb), this);
  gtk_widget_show(tmp_widget);
  fg_emph_color_set_button = tmp_widget;
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  color = prefs.get_fg_strong();
  if (color == NULL)
    {
      color = fg;
      color_set = FALSE;
    }
  else
    color_set = TRUE;
  tmp_widget = gtk_color_button_new_with_color(color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List Foreground Strong Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  fg_strong_color_button = tmp_widget;

  tmp_widget = gtk_check_button_new_with_label(_("Strong"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), color_set);
  g_signal_connect(tmp_widget, "toggled",
		   reinterpret_cast<GCallback>(color_set_toggled_cb), this);
  gtk_widget_show(tmp_widget);
  fg_strong_color_set_button = tmp_widget;
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  color = prefs.get_fg_hidden();
  if (color == NULL)
    {
      color = fg;
      color_set = FALSE;
    }
  else
    color_set = TRUE;
  tmp_widget = gtk_color_button_new_with_color(color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List Foreground Hidden Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  fg_hidden_color_button = tmp_widget;

  tmp_widget = gtk_check_button_new_with_label(_("Hidden"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), color_set);
  g_signal_connect(tmp_widget, "toggled",
		   reinterpret_cast<GCallback>(color_set_toggled_cb), this);
  gtk_widget_show(tmp_widget);
  fg_hidden_color_set_button = tmp_widget;
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  GtkWidget *color_setting_frame
    = gtk_frame_new(_("Foreground Color Settings"));
  gtk_widget_show(hbox);
  gtk_container_add(GTK_CONTAINER(color_setting_frame), hbox);

  gtk_widget_show(color_setting_frame);
  gtk_box_pack_start(GTK_BOX(outer_vbox), color_setting_frame, FALSE, FALSE, 0);



  hbox = gtk_hbutton_box_new();

  vbox = gtk_vbox_new(FALSE, 0);
  color = prefs.get_bg_normal();
  if (color == NULL)
    {
      color = bg;
      color_set = FALSE;
    }
  else
    color_set = TRUE;
  tmp_widget = gtk_color_button_new_with_color(color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List Background Normal Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  bg_normal_color_button = tmp_widget;

  tmp_widget = gtk_check_button_new_with_label(_("Normal"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), color_set);
  g_signal_connect(tmp_widget, "toggled",
		   reinterpret_cast<GCallback>(color_set_toggled_cb), this);
  gtk_widget_show(tmp_widget);
  bg_normal_color_set_button = tmp_widget;
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  color = prefs.get_bg_emph();
  if (color == NULL)
    {
      color = bg;
      color_set = FALSE;
    }
  else
    color_set = TRUE;
  tmp_widget = gtk_color_button_new_with_color(color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List Background Emphasized Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  bg_emph_color_button = tmp_widget;

  tmp_widget = gtk_check_button_new_with_label(_("Emph"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), color_set);
  g_signal_connect(tmp_widget, "toggled",
		   reinterpret_cast<GCallback>(color_set_toggled_cb), this);
  gtk_widget_show(tmp_widget);
  bg_emph_color_set_button = tmp_widget;
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  color = prefs.get_bg_strong();
  if (color == NULL)
    {
      color = bg;
      color_set = FALSE;
    }
  else
    color_set = TRUE;
  tmp_widget = gtk_color_button_new_with_color(color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List Background Strong Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  bg_strong_color_button = tmp_widget;

  tmp_widget = gtk_check_button_new_with_label(_("Strong"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), color_set);
  g_signal_connect(tmp_widget, "toggled",
		   reinterpret_cast<GCallback>(color_set_toggled_cb), this);
  gtk_widget_show(tmp_widget);
  bg_strong_color_set_button = tmp_widget;
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  color = prefs.get_bg_hidden();
  if (color == NULL)
    {
      color = bg;
      color_set = FALSE;
    }
  else
    color_set = TRUE;
  tmp_widget = gtk_color_button_new_with_color(color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List Background Hidden Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  bg_hidden_color_button = tmp_widget;

  tmp_widget = gtk_check_button_new_with_label(_("Hidden"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), color_set);
  g_signal_connect(tmp_widget, "toggled",
		   reinterpret_cast<GCallback>(color_set_toggled_cb), this);
  gtk_widget_show(tmp_widget);
  bg_hidden_color_set_button = tmp_widget;
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);



  color_setting_frame = gtk_frame_new(_("Background Color Settings"));
  gtk_widget_show(hbox);
  gtk_container_add(GTK_CONTAINER(color_setting_frame), hbox);

  gtk_widget_show(color_setting_frame);
  gtk_box_pack_start(GTK_BOX(outer_vbox), color_setting_frame, FALSE, FALSE, 0);


  GtkWidget *frame = gtk_frame_new(_("Settings of List View"));
  gtk_widget_show(outer_vbox);
  gtk_container_add(GTK_CONTAINER(frame), outer_vbox);

  return frame;
}


GtkWidget *
PreferencesUI::create_thread_view_settings_frame()
{
  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);


  GtkWidget *tmp_widget
    = gtk_check_button_new_with_label(_("Enable Popup of Responses"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_enable_popup_response());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.enable_popup_response), tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Enable Popup of Board Name and Thread Title"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_enable_popup_title());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.enable_popup_title), tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Enable Popup of ID"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_enable_popup_id());
  enable_popup_id_button = tmp_widget;
  gtk_widget_set_no_show_all(tmp_widget, TRUE);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.enable_popup_id), tmp_widget);


  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
  tmp_widget = gtk_label_new(_("Delay to Show Popups:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 8);

  GtkAdjustment *adjustment
    = (GtkAdjustment *)gtk_adjustment_new(prefs.get_popup_delay(),
					  0.0, 10000.0, 100.0,
					  1000.0, 1000.0);
  tmp_widget = gtk_spin_button_new(adjustment, 100.0, 0);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
  popup_delay_spin_button = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(changed_cb), this);

  tmp_widget = gtk_label_new(_("(ms)"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);



  hbox = gtk_hbox_new(FALSE, 0);
  tmp_widget = gtk_label_new(_("Delay to Hide Popups:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 8);

  adjustment
    = (GtkAdjustment *)gtk_adjustment_new(prefs.get_popdown_delay(),
					  0.0, 10000.0, 100.0,
					  1000.0, 1000.0);
  tmp_widget = gtk_spin_button_new(adjustment, 100.0, 0);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
  popdown_delay_spin_button = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(changed_cb), this);

  tmp_widget = gtk_label_new(_("(ms)"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);



  tmp_widget = gtk_check_button_new_with_label(_("Show `mailto' Field Literally by Default"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_show_mailto());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.show_mailto), tmp_widget);


  hbox = gtk_hbutton_box_new();

  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&prefs.thread_title_color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Thread Title Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  thread_title_color_button = tmp_widget;

  tmp_widget = gtk_label_new(_("Thread Title"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&prefs.name_color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Name Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  name_color_button = tmp_widget;

  tmp_widget = gtk_label_new(_("Name"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&prefs.link_color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Link Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  link_color_button = tmp_widget;

  tmp_widget = gtk_label_new(_("Link"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&prefs.visited_color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Visited Link Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  visited_color_button = tmp_widget;

  tmp_widget = gtk_label_new(_("Visited Link"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&prefs.auto_refresh_go_color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Auto Refresh Go Label Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  auto_refresh_go_color_button = tmp_widget;

  tmp_widget = gtk_label_new(_("Go"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
  auto_refresh_go = vbox;


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&prefs.auto_refresh_wait_color);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Auto Refresh Wait Label Color"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  auto_refresh_wait_color_button = tmp_widget;

  tmp_widget = gtk_label_new(_("Wait"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
  auto_refresh_wait = vbox;


  GtkWidget *color_setting_frame = gtk_frame_new(_("Color Settings"));
  gtk_widget_show(hbox);
  gtk_container_add(GTK_CONTAINER(color_setting_frame), hbox);

  gtk_widget_show(color_setting_frame);
  gtk_box_pack_end(GTK_BOX(outer_vbox), color_setting_frame, FALSE, FALSE, 0);


  GtkWidget *frame = gtk_frame_new(_("Settings of Thread View"));
  gtk_widget_show(outer_vbox);
  gtk_container_add(GTK_CONTAINER(frame), outer_vbox);

  return frame;
}


GtkWidget *
PreferencesUI::create_a_bone_settings_frame()
{
  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);


  GtkWidget *tmp_widget
    = gtk_check_button_new_with_label(_("Enable Transparent A Bone"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_enable_transparent_a_bone());
  //enable_transparent_a_bone_button = tmp_widget;
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.enable_transparent_a_bone),
			   tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Disable Popup of Response made A Bone"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_disable_popup_a_bone());
  //disable_popup_a_bone_button = tmp_widget;
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.disable_popup_a_bone), tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Disable Local a Bone"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_disable_a_bone());
  //disable_a_bone_button = tmp_widget;
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.disable_a_bone), tmp_widget);


  tmp_widget = gtk_check_button_new_with_label(_("Enable Chain a Bone"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_enable_chain_a_bone());
  //enable_chain_a_bone_button = tmp_widget;
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.enable_chain_a_bone), tmp_widget);


  a_bone_settings = new ABoneSettings(config.get_a_bone());
  GtkWidget *widget = a_bone_settings->get_widget();
  gtk_widget_show(widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), widget, TRUE, TRUE, 0);

  GtkWidget *frame = gtk_frame_new(_("Settings of A Bone"));
  gtk_widget_show(outer_vbox);
  gtk_container_add(GTK_CONTAINER(frame), outer_vbox);

  return frame;
}


GtkWidget *
PreferencesUI::create_image_view_settings_frame()
{
  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);

  GtkWidget *tmp_widget = gtk_check_button_new_with_label(_("Enable Inline Preview of Images"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_enable_inline_image());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.enable_inline_image), tmp_widget);


  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);

  tmp_widget = gtk_label_new(_("Maximum Inline Image Size:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 8);

  tmp_widget = gtk_label_new(_("Width:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);

  GtkAdjustment *adjustment
    = reinterpret_cast<GtkAdjustment *>(gtk_adjustment_new(prefs.get_inline_image_width(),
							   1, 1600, 1, 32, 0));
  tmp_widget = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 16.0, 0);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
  inline_image_width_spin_button = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(changed_cb), this);


  tmp_widget = gtk_label_new(_("x Height:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);

  adjustment
    = (GtkAdjustment *)gtk_adjustment_new(prefs.get_inline_image_height(),
					  1, 1200, 1, 32, 0);
  tmp_widget = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 16.0, 0);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
  inline_image_height_spin_button = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(changed_cb), this);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);


  tmp_widget = gtk_check_button_new_with_label(_("Enable Popup Preview of Images"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_enable_popup_image());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.enable_popup_image), tmp_widget);


  hbox = gtk_hbox_new(FALSE, 0);

  tmp_widget = gtk_label_new(_("Maximum Popup Image Size:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 8);

  tmp_widget = gtk_label_new(_("Width:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);

  adjustment
    = reinterpret_cast<GtkAdjustment *>(gtk_adjustment_new(prefs.get_popup_image_width(),
							   1, 1600, 1, 32, 0));
  tmp_widget = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 16.0, 0);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
  popup_image_width_spin_button = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(changed_cb), this);


  tmp_widget = gtk_label_new(_("x Height:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);

  adjustment
    = reinterpret_cast<GtkAdjustment *>(gtk_adjustment_new(prefs.get_popup_image_height(),
							   1, 1200, 1, 32, 0));
  tmp_widget = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 16.0, 0);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
  popup_image_height_spin_button = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(changed_cb), this);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);



  GtkWidget *inner_vbox = gtk_vbox_new(FALSE, 0);

  hbox = gtk_hbox_new(FALSE, 0);

  tmp_widget = gtk_label_new(_("Maximum Number of Image Cache Files:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 8);

  adjustment
    = reinterpret_cast<GtkAdjustment *>(gtk_adjustment_new(prefs.get_maximum_number_of_image_cache_files(),
							   0, 0x100000, 256, 1024, 0));
  tmp_widget = gtk_spin_button_new(adjustment, 16.0, 0);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
  maximum_number_of_image_cache_files_spin_button = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(changed_cb), this);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(inner_vbox), hbox, FALSE, FALSE, 0);



  hbox = gtk_hbox_new(FALSE, 0);

  tmp_widget = gtk_label_new(_("Capacity of Image Cache:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 8);

  adjustment
    = reinterpret_cast<GtkAdjustment *>(gtk_adjustment_new(prefs.get_capacity_of_image_cache(),
							   0, 0x100000,
							   32, 1024, 0));
  tmp_widget = gtk_spin_button_new(adjustment, 16.0, 0);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
  capacity_of_image_cache_spin_button = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(changed_cb), this);

  tmp_widget = gtk_label_new(_("(MB)"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(inner_vbox), hbox, FALSE, FALSE, 0);



  GtkWidget *inner_frame = gtk_frame_new(_("Settings of Image Cache"));
  gtk_widget_show(inner_vbox);
  gtk_container_add(GTK_CONTAINER(inner_frame), inner_vbox);

  gtk_widget_show(inner_frame);
  gtk_box_pack_end(GTK_BOX(outer_vbox), inner_frame, FALSE, FALSE, 0);

  GtkWidget *frame = gtk_frame_new(_("Settings of Image View"));
  gtk_widget_show(outer_vbox);
  gtk_container_add(GTK_CONTAINER(frame), outer_vbox);

  return frame;
}


GtkWidget *
PreferencesUI::create_my_bookmark_settings_frame()
{
  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);

  GtkWidget *tmp_widget
    = gtk_check_button_new_with_label(_("Do NOT Show My Bookmark"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_hide_bookmark());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_action_connect_proxy(GTK_ACTION(prefs.hide_bookmark), tmp_widget);


  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);

  tmp_widget = gtk_label_new(_("My Bookmark"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 4);


  GtkWidget *scrolled = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  bookmark_text_buffer = gtk_text_buffer_new(NULL);
  GtkTextIter iter;
  gtk_text_buffer_get_start_iter(bookmark_text_buffer, &iter);
  gtk_text_buffer_insert(bookmark_text_buffer, &iter,
			 prefs.get_bookmark_text(), -1);
  g_signal_connect(bookmark_text_buffer, "changed",
		   reinterpret_cast<GCallback>(changed_cb), this);

  tmp_widget = gtk_text_view_new_with_buffer(bookmark_text_buffer);
  gtk_widget_set_name(tmp_widget, "mybookmarkeditor");
  gtk_widget_show(tmp_widget);
  gtk_container_add(GTK_CONTAINER(scrolled), tmp_widget);
  gtk_widget_show(scrolled);
  gtk_box_pack_start(GTK_BOX(outer_vbox), scrolled, TRUE, TRUE, 0);


  GtkWidget *frame = gtk_frame_new(_("Settings of My Bookmark"));
  gtk_widget_show(outer_vbox);
  gtk_container_add(GTK_CONTAINER(frame), outer_vbox);

  return frame;
}


static void
ensure_5digits_only(GtkEntry *entry)
{
  const gchar *text = gtk_entry_get_text(entry);
  if (text == NULL)	// ͭʤɬ""뤫⡩
    text = "";

  int val = 0;
  if (sscanf(text, "%d", &val) != 1)
    val = 0;

  char buf[16];
  snprintf(buf, 16, "%d", (val & 0xffff));
  if (strcmp(text, buf) == 0)
    return;

  gtk_entry_set_text(entry, buf);
  gtk_editable_set_position(GTK_EDITABLE(entry), -1);
}


gboolean
PreferencesUI::ensure_5digits_only_on_idle(PreferencesUI *ui)
{
  gdk_threads_enter();

  ensure_5digits_only(GTK_ENTRY(ui->proxy_port_to_read_entry));
  ensure_5digits_only(GTK_ENTRY(ui->proxy_port_to_post_entry));
  ensure_5digits_only(GTK_ENTRY(ui->proxy_port_for_images_entry));

  ui->ensure_5digits_only_on_idle_id = 0;

  gdk_threads_leave();

  return FALSE;
}


void
PreferencesUI::port_entry_changed_cb(GtkEntry *entry, PreferencesUI *ui)
{
  if (ui->ensure_5digits_only_on_idle_id == 0)
    ui->ensure_5digits_only_on_idle_id
      = g_idle_add(reinterpret_cast<GSourceFunc>(ensure_5digits_only_on_idle),
		   ui);
  return;
}


static void
setup_proxy_entries(const URI &uri,
		    GtkWidget *server_entry, GtkWidget *port_entry,
		    GtkWidget *user_entry, GtkWidget *passwd_entry)
{
  if (!uri.is_valid())
    {
      gtk_entry_set_text(GTK_ENTRY(server_entry), "");
      gtk_entry_set_text(GTK_ENTRY(port_entry), "0");
      gtk_entry_set_text(GTK_ENTRY(user_entry), "");
      gtk_entry_set_text(GTK_ENTRY(passwd_entry), "");
      return;
    }

  const char *server = uri.get_server();
  if (server != NULL)
    gtk_entry_set_text(GTK_ENTRY(server_entry), server);

  int port = uri.get_port();
  char buf[16];
  snprintf(buf, 16, "%d", port & 0xffff);
  gtk_entry_set_text(GTK_ENTRY(port_entry), buf);

  char *user_passwd;
  const char *user;
  const char *passwd;
  const char *src_user = uri.get_user();
  if (src_user != NULL)
    {
      user_passwd = strdup(src_user);
      user = user_passwd;
      char *colon_pos = strchr(user_passwd, ':');
      if (colon_pos != NULL)
	{
	  passwd = colon_pos + 1;
	  *colon_pos = '\0';
	}
      else
	passwd = NULL;

      gtk_entry_set_text(GTK_ENTRY(user_entry), user);
      if (passwd != NULL)
	gtk_entry_set_text(GTK_ENTRY(passwd_entry), user);

      free(user_passwd);
    }
}


GtkWidget *
PreferencesUI::create_network_settings_frame()
{
  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);

  // ̾ɤ߹proxy
  GtkWidget *tmp_widget = gtk_check_button_new_with_label(_("Use Proxy Server to Read"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_enable_proxy_to_read());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 3);
  gtk_action_connect_proxy(GTK_ACTION(prefs.enable_proxy_to_read), tmp_widget);

  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);


  tmp_widget = gtk_label_new(_("Server:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);

  tmp_widget = gtk_entry_new();
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 3);
  proxy_server_to_read_entry = tmp_widget;


  tmp_widget = gtk_label_new(_("Port:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);

  tmp_widget = gtk_entry_new();
  gtk_entry_set_width_chars(GTK_ENTRY(tmp_widget), 5);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);
  proxy_port_to_read_entry = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(port_entry_changed_cb), this);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);


  hbox = gtk_hbox_new(FALSE, 0);

  tmp_widget = gtk_label_new(_("User:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);

  tmp_widget = gtk_entry_new();
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 3);
  proxy_user_to_read_entry = tmp_widget;


  tmp_widget = gtk_label_new(_("Passwd:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);

  tmp_widget = gtk_entry_new();
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 3);
  proxy_passwd_to_read_entry = tmp_widget;
  gtk_entry_set_visibility(GTK_ENTRY(tmp_widget), FALSE);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);


  setup_proxy_entries(prefs.get_proxy_uri_to_read(),
		      proxy_server_to_read_entry, proxy_port_to_read_entry,
		      proxy_user_to_read_entry, proxy_passwd_to_read_entry);


  // proxy
  tmp_widget = gtk_hseparator_new();
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 6);

  tmp_widget = gtk_check_button_new_with_label(_("Use Proxy Server to Post"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_enable_proxy_to_post());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 3);
  gtk_action_connect_proxy(GTK_ACTION(prefs.enable_proxy_to_post), tmp_widget);

  hbox = gtk_hbox_new(FALSE, 0);


  tmp_widget = gtk_label_new(_("Server:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);

  tmp_widget = gtk_entry_new();
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 3);
  proxy_server_to_post_entry = tmp_widget;


  tmp_widget = gtk_label_new(_("Port:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);

  tmp_widget = gtk_entry_new();
  gtk_entry_set_width_chars(GTK_ENTRY(tmp_widget), 5);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);
  proxy_port_to_post_entry = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(port_entry_changed_cb), this);


  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);


  hbox = gtk_hbox_new(FALSE, 0);

  tmp_widget = gtk_label_new(_("User:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);

  tmp_widget = gtk_entry_new();
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 3);
  proxy_user_to_post_entry = tmp_widget;

  tmp_widget = gtk_label_new(_("Passwd:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);

  tmp_widget = gtk_entry_new();
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 3);
  proxy_passwd_to_post_entry = tmp_widget;
  gtk_entry_set_visibility(GTK_ENTRY(tmp_widget), FALSE);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);


  setup_proxy_entries(prefs.get_proxy_uri_to_post(),
		      proxy_server_to_post_entry, proxy_port_to_post_entry,
		      proxy_user_to_post_entry, proxy_passwd_to_post_entry);


  // proxy
  tmp_widget = gtk_hseparator_new();
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 6);

  tmp_widget
    = gtk_check_button_new_with_label(_("Use Proxy Server for Images"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
			       prefs.get_enable_proxy_for_images());
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 3);
  gtk_action_connect_proxy(GTK_ACTION(prefs.enable_proxy_for_images),
			   tmp_widget);

  hbox = gtk_hbox_new(FALSE, 0);


  tmp_widget = gtk_label_new(_("Server:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);

  tmp_widget = gtk_entry_new();
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 3);
  proxy_server_for_images_entry = tmp_widget;


  tmp_widget = gtk_label_new(_("Port:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);

  tmp_widget = gtk_entry_new();
  gtk_entry_set_width_chars(GTK_ENTRY(tmp_widget), 5);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);
  proxy_port_for_images_entry = tmp_widget;
  g_signal_connect(tmp_widget, "changed",
		   reinterpret_cast<GCallback>(port_entry_changed_cb), this);


  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);


  hbox = gtk_hbox_new(FALSE, 0);

  tmp_widget = gtk_label_new(_("User:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);

  tmp_widget = gtk_entry_new();
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 3);
  proxy_user_for_images_entry = tmp_widget;

  tmp_widget = gtk_label_new(_("Passwd:"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 3);

  tmp_widget = gtk_entry_new();
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 3);
  proxy_passwd_for_images_entry = tmp_widget;
  gtk_entry_set_visibility(GTK_ENTRY(tmp_widget), FALSE);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);


  setup_proxy_entries(prefs.get_proxy_uri_for_images(),
		      proxy_server_for_images_entry,
		      proxy_port_for_images_entry,
		      proxy_user_for_images_entry,
		      proxy_passwd_for_images_entry);


  GtkWidget *warning_vbox = gtk_vbox_new(FALSE, 0);

  hbox = gtk_hbox_new(FALSE, 0);
  tmp_widget = gtk_label_new(_("WARNING: Use of PROXY has tested very little.  If this doesn't work well for you, please report."));
  gtk_label_set_justify(GTK_LABEL(tmp_widget), GTK_JUSTIFY_LEFT);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(warning_vbox), hbox, FALSE, FALSE, 4);

  hbox = gtk_hbox_new(FALSE, 0);
  tmp_widget = gtk_label_new(_("WARNING: We only support non-authentication or the `Basic Authentication' at this time."));
  gtk_label_set_justify(GTK_LABEL(tmp_widget), GTK_JUSTIFY_LEFT);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(warning_vbox), hbox, FALSE, FALSE, 4);

  hbox = gtk_hbox_new(FALSE, 0);
  tmp_widget = gtk_label_new(_("WARNING: Sorry, I cannot recommend to use PROXY at this time."));
  gtk_label_set_justify(GTK_LABEL(tmp_widget), GTK_JUSTIFY_LEFT);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 4);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(warning_vbox), hbox, FALSE, FALSE, 4);


  // XXX
  GtkWidget *proxy_frame = gtk_frame_new(_("Warnings about Proxy Settings"));
  gtk_widget_show(warning_vbox);
  gtk_container_add(GTK_CONTAINER(proxy_frame), warning_vbox);

  gtk_widget_show(proxy_frame);
  gtk_box_pack_end(GTK_BOX(outer_vbox), proxy_frame, FALSE, FALSE, 0);

  GtkWidget *frame = gtk_frame_new(_("Settings of Network"));
  gtk_widget_show(outer_vbox);
  gtk_container_add(GTK_CONTAINER(frame), outer_vbox);

  return frame;
}


GtkWidget *
PreferencesUI::create_state_color_settings_frame()
{
  GtkWidget *scrolled = controller.get_boardlist_ui().get_widget();
  GtkStyle *style = NULL;
  if (scrolled != NULL && GTK_IS_SCROLLED_WINDOW(scrolled))
    {
      GtkWidget *tree_view = gtk_bin_get_child(GTK_BIN(scrolled));
      if (tree_view != NULL)
	style = gtk_rc_get_style(tree_view);
    }
  g_return_val_if_fail(style != NULL, NULL);


  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);


  GtkWidget *hbox = gtk_hbutton_box_new();

  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
  GtkWidget *tmp_widget
    = gtk_color_button_new_with_color(&style->fg[GTK_STATE_NORMAL]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Foreground Color on NORMAL state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  fg_normal_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("NORMAL");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&style->fg[GTK_STATE_PRELIGHT]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Foreground Color on PRELIGHT state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  fg_prelight_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("PRELIGHT");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&style->fg[GTK_STATE_ACTIVE]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Foreground Color on ACTIVE state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  fg_active_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("ACTIVE");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&style->fg[GTK_STATE_SELECTED]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Foreground Color on SELECTED state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  fg_selected_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("SELECTED");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget
    = gtk_color_button_new_with_color(&style->fg[GTK_STATE_INSENSITIVE]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Foreground Color on INSENSITIVE state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  fg_insensitive_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("INSENSITIVE");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);

  GtkWidget *inner_frame = gtk_frame_new(_("Foreground Color Settings"));
  gtk_widget_show(hbox);
  gtk_container_add(GTK_CONTAINER(inner_frame), hbox);

  gtk_widget_show(inner_frame);
  gtk_box_pack_start(GTK_BOX(outer_vbox), inner_frame, FALSE, FALSE, 0);



  hbox = gtk_hbutton_box_new();

  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&style->bg[GTK_STATE_NORMAL]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Background Color on NORMAL state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  bg_normal_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("NORMAL");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&style->bg[GTK_STATE_PRELIGHT]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Background Color on PRELIGHT state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  bg_prelight_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("PRELIGHT");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&style->bg[GTK_STATE_ACTIVE]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Background Color on ACTIVE state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  bg_active_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("ACTIVE");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&style->bg[GTK_STATE_SELECTED]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Background Color on SELECTED state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  bg_selected_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("SELECTED");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget
    = gtk_color_button_new_with_color(&style->bg[GTK_STATE_INSENSITIVE]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("Background Color on INSENSITIVE state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  bg_insensitive_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("INSENSITIVE");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  inner_frame = gtk_frame_new(_("Background Color Settings"));
  gtk_widget_show(hbox);
  gtk_container_add(GTK_CONTAINER(inner_frame), hbox);

  gtk_widget_show(inner_frame);
  gtk_box_pack_start(GTK_BOX(outer_vbox), inner_frame, FALSE, FALSE, 0);



  hbox = gtk_hbutton_box_new();

  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&style->text[GTK_STATE_NORMAL]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List/Tree/Text Foreground Color on NORMAL state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  text_normal_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("NORMAL");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget
    = gtk_color_button_new_with_color(&style->text[GTK_STATE_PRELIGHT]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List/Tree/Text Foreground Color on PRELIGHT state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  text_prelight_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("PRELIGHT");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&style->text[GTK_STATE_ACTIVE]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List/Tree/Text Foreground Color on ACTIVE state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  text_active_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("ACTIVE");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget
    = gtk_color_button_new_with_color(&style->text[GTK_STATE_SELECTED]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List/Tree/Text Foreground Color on SELECTED state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  text_selected_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("SELECTED");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget
    = gtk_color_button_new_with_color(&style->text[GTK_STATE_INSENSITIVE]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List/Tree/Text Foreground Color on INSENSITIVE state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  text_insensitive_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("INSENSITIVE");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);

  inner_frame = gtk_frame_new(_("List/Tree/Text Foreground Color Settings"));
  gtk_widget_show(hbox);
  gtk_container_add(GTK_CONTAINER(inner_frame), hbox);

  gtk_widget_show(inner_frame);
  gtk_box_pack_start(GTK_BOX(outer_vbox), inner_frame, FALSE, FALSE, 0);



  hbox = gtk_hbutton_box_new();

  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&style->base[GTK_STATE_NORMAL]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List/Tree/Text Background Color on NORMAL state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  base_normal_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("NORMAL");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget
    = gtk_color_button_new_with_color(&style->base[GTK_STATE_PRELIGHT]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List/Tree/Text Background Color on PRELIGHT state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  base_prelight_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("PRELIGHT");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&style->base[GTK_STATE_ACTIVE]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List/Tree/Text Background Color on ACTIVE state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  base_active_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("ACTIVE");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget = gtk_color_button_new_with_color(&style->base[GTK_STATE_SELECTED]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List/Tree/Text Background Color on SELECTED state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  base_selected_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("SELECTED");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);


  vbox = gtk_vbox_new(FALSE, 0);
  tmp_widget
    = gtk_color_button_new_with_color(&style->base[GTK_STATE_INSENSITIVE]);
  g_signal_connect(tmp_widget, "color-set",
		   reinterpret_cast<GCallback>(state_color_set_cb), this);
  gtk_color_button_set_title(GTK_COLOR_BUTTON(tmp_widget),
			     _("List/Tree/Text Background Color on INSENSITIVE state"));
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  base_insensitive_state_color_button = tmp_widget;

  tmp_widget = gtk_label_new("INSENSITIVE");
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);

  inner_frame = gtk_frame_new(_("List/Tree/Text Background Color Settings"));
  gtk_widget_show(hbox);
  gtk_container_add(GTK_CONTAINER(inner_frame), hbox);

  gtk_widget_show(inner_frame);
  gtk_box_pack_start(GTK_BOX(outer_vbox), inner_frame, FALSE, FALSE, 0);


  GtkWidget *frame = gtk_frame_new(_("Settings of Colors"));
  gtk_widget_show(outer_vbox);
  gtk_container_add(GTK_CONTAINER(frame), outer_vbox);

  return frame;
}


void
PreferencesUI::color_set_cb(GtkColorButton *button, PreferencesUI *ui)
{
  GtkWidget *widget = reinterpret_cast<GtkWidget *>(button);
  GdkColor *color;

  if (widget == ui->fg_normal_color_button)
    color = &ui->prefs.fg_normal_color;
  else if (widget == ui->fg_emph_color_button)
    color = &ui->prefs.fg_emph_color;
  else if (widget == ui->fg_strong_color_button)
    color = &ui->prefs.fg_strong_color;
  else if (widget == ui->fg_hidden_color_button)
    color = &ui->prefs.fg_hidden_color;
  else if (widget == ui->bg_normal_color_button)
    color = &ui->prefs.bg_normal_color;
  else if (widget == ui->bg_emph_color_button)
    color = &ui->prefs.bg_emph_color;
  else if (widget == ui->bg_strong_color_button)
    color = &ui->prefs.bg_strong_color;
  else if (widget == ui->bg_hidden_color_button)
    color = &ui->prefs.bg_hidden_color;
  else if (widget == ui->thread_title_color_button)
    color = &ui->prefs.thread_title_color;
  else if (widget == ui->name_color_button)
    color = &ui->prefs.name_color;
  else if (widget == ui->link_color_button)
    color = &ui->prefs.link_color;
  else if (widget == ui->visited_color_button)
    color = &ui->prefs.visited_color;
  else if (widget == ui->auto_refresh_go_color_button)
    color = &ui->prefs.auto_refresh_go_color;
  else if (widget == ui->auto_refresh_wait_color_button)
    color = &ui->prefs.auto_refresh_wait_color;
  else
    color = NULL;

  g_return_if_fail(color != NULL);

  gtk_color_button_get_color(button, color);

  gdk_threads_leave();

  ui->prefs.prefs_changed(ui->prefs);

  gdk_threads_enter();
}


void
PreferencesUI::color_set_toggled_cb(GtkToggleButton *button, PreferencesUI *ui)
{
  GtkWidget *widget = reinterpret_cast<GtkWidget *>(button);
  gboolean active = gtk_toggle_button_get_active(button);

  if (widget == ui->fg_normal_color_set_button)
    ui->prefs.set_fg_normal_color_set(active);
  else if (widget == ui->fg_emph_color_set_button)
    ui->prefs.set_fg_emph_color_set(active);
  else if (widget == ui->fg_strong_color_set_button)
    ui->prefs.set_fg_strong_color_set(active);
  else if (widget == ui->fg_hidden_color_set_button)
    ui->prefs.set_fg_hidden_color_set(active);
  else if (widget == ui->bg_normal_color_set_button)
    ui->prefs.set_bg_normal_color_set(active);
  else if (widget == ui->bg_emph_color_set_button)
    ui->prefs.set_bg_emph_color_set(active);
  else if (widget == ui->bg_strong_color_set_button)
    ui->prefs.set_bg_strong_color_set(active);
  else if (widget == ui->bg_hidden_color_set_button)
    ui->prefs.set_bg_hidden_color_set(active);
}


void
PreferencesUI::state_color_set_cb(GtkColorButton *button, PreferencesUI *ui)
{
  ui->write_anarchy_gtkrc();
  ui->reload_anarchy_gtkrc();
}


void
PreferencesUI::changed_cb(GObject *object, PreferencesUI *ui)
{
  // applyܥsensitiveˤ롣
  g_return_if_fail(ui->dialog != NULL);
  gtk_dialog_set_response_sensitive(GTK_DIALOG(ui->dialog),
				    GTK_RESPONSE_APPLY, TRUE);
}


void
PreferencesUI::response_cb(GtkWidget *dialog, int response_id,
			   PreferencesUI *ui)
{
  switch (response_id)
    {
    case GTK_RESPONSE_APPLY:
    case GTK_RESPONSE_OK:
      {
	GtkEntry *entry = GTK_ENTRY(ui->web_browser_template_entry);
	const gchar *text = gtk_entry_get_text(entry);
	ui->prefs.set_web_browser_template(text);
	gtk_entry_set_text(entry, ui->prefs.get_web_browser_template());

	entry = GTK_ENTRY(ui->downloader_template_entry);
	text = gtk_entry_get_text(entry);
	ui->prefs.set_downloader_template(text);
	gtk_entry_set_text(entry, ui->prefs.get_downloader_template());

	GtkSpinButton *spin = GTK_SPIN_BUTTON(ui->popup_delay_spin_button);
	ui->prefs.set_popup_delay(gtk_spin_button_get_value_as_int(spin));

	spin = GTK_SPIN_BUTTON(ui->popdown_delay_spin_button);
	ui->prefs.set_popdown_delay(gtk_spin_button_get_value_as_int(spin));

	spin = GTK_SPIN_BUTTON(ui->inline_image_width_spin_button);
	ui->prefs.set_inline_image_width(gtk_spin_button_get_value_as_int(spin));

	spin = GTK_SPIN_BUTTON(ui->inline_image_height_spin_button);
	ui->prefs.set_inline_image_height(gtk_spin_button_get_value_as_int(spin));

	spin = GTK_SPIN_BUTTON(ui->popup_image_width_spin_button);
	ui->prefs.set_popup_image_width(gtk_spin_button_get_value_as_int(spin));

	spin = GTK_SPIN_BUTTON(ui->popup_image_height_spin_button);
	ui->prefs.set_popup_image_height(gtk_spin_button_get_value_as_int(spin));

	spin = GTK_SPIN_BUTTON(ui->maximum_number_of_image_cache_files_spin_button);
	ui->prefs.set_maximum_number_of_image_cache_files(gtk_spin_button_get_value_as_int(spin));

	spin = GTK_SPIN_BUTTON(ui->capacity_of_image_cache_spin_button);
	ui->prefs.set_capacity_of_image_cache(gtk_spin_button_get_value_as_int(spin));

	GtkTextIter start_iter;
	GtkTextIter end_iter;
	gtk_text_buffer_get_start_iter(ui->bookmark_text_buffer, &start_iter);
	gtk_text_buffer_get_end_iter(ui->bookmark_text_buffer, &end_iter);
	gchar *bookmark_text
	  = gtk_text_buffer_get_text(ui->bookmark_text_buffer,
				     &start_iter, &end_iter, FALSE);
	gtk_text_buffer_delete(ui->bookmark_text_buffer,
			       &start_iter, &end_iter);
	ui->prefs.set_bookmark_text(bookmark_text);
	g_free(bookmark_text);
	gtk_text_buffer_insert(ui->bookmark_text_buffer, &start_iter,
			       ui->prefs.get_bookmark_text(), -1);


	int port;
	text = gtk_entry_get_text(GTK_ENTRY(ui->proxy_port_to_read_entry));
	if (text != NULL)
	  {
	    if (sscanf(text, "%d", &port) != 1)
	      port = 0;
	  }
	else
	  port = 0;

	ui->prefs.set_proxy_to_read(
		gtk_entry_get_text(GTK_ENTRY(ui->proxy_server_to_read_entry)),
		port,
		gtk_entry_get_text(GTK_ENTRY(ui->proxy_user_to_read_entry)),
		gtk_entry_get_text(GTK_ENTRY(ui->proxy_passwd_to_read_entry)));


	text = gtk_entry_get_text(GTK_ENTRY(ui->proxy_port_to_post_entry));
	if (text != NULL)
	  {
	    if (sscanf(text, "%d", &port) != 1)
	      port = 0;
	  }
	else
	  port = 0;

	ui->prefs.set_proxy_to_post(
		gtk_entry_get_text(GTK_ENTRY(ui->proxy_server_to_post_entry)),
		port,
		gtk_entry_get_text(GTK_ENTRY(ui->proxy_user_to_post_entry)),
		gtk_entry_get_text(GTK_ENTRY(ui->proxy_passwd_to_post_entry)));


	text = gtk_entry_get_text(GTK_ENTRY(ui->proxy_port_for_images_entry));
	if (text != NULL)
	  {
	    if (sscanf(text, "%d", &port) != 1)
	      port = 0;
	  }
	else
	  port = 0;

	ui->prefs.set_proxy_for_images(
		gtk_entry_get_text(GTK_ENTRY(ui->proxy_server_for_images_entry)),
		port,
		gtk_entry_get_text(GTK_ENTRY(ui->proxy_user_for_images_entry)),
		gtk_entry_get_text(GTK_ENTRY(ui->proxy_passwd_for_images_entry)));

	break;
      }

    default:
      {
      // canceldeleteʤɤξ硢᤹
	GtkEntry *entry = GTK_ENTRY(ui->web_browser_template_entry);
	gtk_entry_set_text(entry, ui->prefs.get_web_browser_template());

	entry = GTK_ENTRY(ui->downloader_template_entry);
	gtk_entry_set_text(entry, ui->prefs.get_downloader_template());

	GtkSpinButton *spin = GTK_SPIN_BUTTON(ui->popup_delay_spin_button);

	spin = GTK_SPIN_BUTTON(ui->popdown_delay_spin_button);
	gtk_spin_button_set_value(spin, ui->prefs.get_popdown_delay());

	spin = GTK_SPIN_BUTTON(ui->inline_image_width_spin_button);
	gtk_spin_button_set_value(spin, ui->prefs.get_inline_image_width());

	spin = GTK_SPIN_BUTTON(ui->inline_image_height_spin_button);
	gtk_spin_button_set_value(spin, ui->prefs.get_inline_image_height());

	spin = GTK_SPIN_BUTTON(ui->popup_image_width_spin_button);
	gtk_spin_button_set_value(spin, ui->prefs.get_popup_image_width());

	spin = GTK_SPIN_BUTTON(ui->popup_image_height_spin_button);
	gtk_spin_button_set_value(spin, ui->prefs.get_popup_image_height());

	spin = GTK_SPIN_BUTTON(ui->maximum_number_of_image_cache_files_spin_button);
	gtk_spin_button_set_value(spin,
				  ui->prefs.get_maximum_number_of_image_cache_files());

	spin = GTK_SPIN_BUTTON(ui->capacity_of_image_cache_spin_button);
	gtk_spin_button_set_value(spin,
				  ui->prefs.get_capacity_of_image_cache());

	GtkTextIter start_iter;
	GtkTextIter end_iter;
	gtk_text_buffer_get_start_iter(ui->bookmark_text_buffer, &start_iter);
	gtk_text_buffer_get_end_iter(ui->bookmark_text_buffer, &end_iter);
	gtk_text_buffer_delete(ui->bookmark_text_buffer,
			       &start_iter, &end_iter);
	gtk_text_buffer_insert(ui->bookmark_text_buffer, &start_iter,
			       ui->prefs.get_bookmark_text(), -1);

	setup_proxy_entries(ui->prefs.get_proxy_uri_to_read(),
			    ui->proxy_server_to_read_entry,
			    ui->proxy_port_to_read_entry,
			    ui->proxy_user_to_read_entry,
			    ui->proxy_passwd_to_read_entry);

	setup_proxy_entries(ui->prefs.get_proxy_uri_to_post(),
			    ui->proxy_server_to_post_entry,
			    ui->proxy_port_to_post_entry,
			    ui->proxy_user_to_post_entry,
			    ui->proxy_passwd_to_post_entry);

	setup_proxy_entries(ui->prefs.get_proxy_uri_for_images(),
			    ui->proxy_server_for_images_entry,
			    ui->proxy_port_for_images_entry,
			    ui->proxy_user_for_images_entry,
			    ui->proxy_passwd_for_images_entry);

	break;
      }
    }

  gtk_dialog_set_response_sensitive(GTK_DIALOG(ui->dialog),
				    GTK_RESPONSE_APPLY, FALSE);

  if (response_id == GTK_RESPONSE_APPLY)
    return;

  gtk_widget_hide(dialog);
}


void
PreferencesUI::show_all_toggled_cb(GtkToggleButton *button,
				   PreferencesUI *ui)
{
  gboolean active = gtk_toggle_button_get_active(button);
  if (active)
    {
      gtk_widget_show(ui->focus_follow_mouse_button);
      gtk_widget_show(ui->default_open_in_tab_button);
      gtk_widget_show(ui->enable_popup_id_button);

      if (ui->a_bone_settings_frame != NULL)
	gtk_widget_show(ui->a_bone_settings_frame);

      gtk_widget_show(ui->auto_refresh_go);
      gtk_widget_show(ui->auto_refresh_wait);

      if (ui->state_color_settings_frame != NULL)
	gtk_widget_show(ui->state_color_settings_frame);
    }
  else
    {
      gtk_widget_hide(ui->focus_follow_mouse_button);
      gtk_widget_hide(ui->default_open_in_tab_button);
      gtk_widget_hide(ui->enable_popup_id_button);

      if (ui->a_bone_settings_frame != NULL)
	gtk_widget_hide(ui->a_bone_settings_frame);

      gtk_widget_hide(ui->auto_refresh_go);
      gtk_widget_hide(ui->auto_refresh_wait);

      if (ui->state_color_settings_frame != NULL)
	gtk_widget_hide(ui->state_color_settings_frame);
    }

  SessionClient &session = ui->controller.get_session();
  char value[128];
  snprintf(value, 128, "%u", active);
  session.set_state("PreferencesUI::show_all", value);
}


#define OUTPUT_COLOR_SPEC(group, STATE, state)				\
  do									\
    {									\
      gtk_color_button_get_color(GTK_COLOR_BUTTON(group##_##state##_state_color_button), \
				 &color);				\
      fprintf(file, "  " #group "[" #STATE "]=\"#%02x%02x%02x\"\n",	\
	      (color.red >> 8) & 0xff, (color.green >> 8) & 0xff,	\
	      (color.blue >> 8) & 0xff);				\
    }									\
  while (0)


void
PreferencesUI::write_anarchy_gtkrc()
{
  int fd = controller.get_repository().open_file(OCHUSHA_ANARCHY_GTKRC,
						 O_WRONLY | O_TRUNC | O_CREAT);
  g_return_if_fail(fd >= 0);
  FILE *file = fdopen(fd, "w");
  if (file == NULL)
    {
      close(fd);
      g_return_if_fail(file != NULL);
    }

  fprintf(file, "# " OCHUSHA_ANARCHY_GTKRC " generated by "
	  PACKAGE_STRING ".\n");
  fprintf(file, "# Do NOT Edit!\n\n");
  fprintf(file, "style \"ochusha-common-style\"\n{\n");

  GdkColor color;

  OUTPUT_COLOR_SPEC(fg, NORMAL, normal);
  OUTPUT_COLOR_SPEC(bg, NORMAL, normal);
  OUTPUT_COLOR_SPEC(fg, PRELIGHT, prelight);
  OUTPUT_COLOR_SPEC(bg, PRELIGHT, prelight);
  OUTPUT_COLOR_SPEC(fg, ACTIVE, active);
  OUTPUT_COLOR_SPEC(bg, ACTIVE, active);
  OUTPUT_COLOR_SPEC(fg, SELECTED, selected);
  OUTPUT_COLOR_SPEC(bg, SELECTED, selected);
  OUTPUT_COLOR_SPEC(fg, INSENSITIVE, insensitive);
  OUTPUT_COLOR_SPEC(bg, INSENSITIVE, insensitive);

  OUTPUT_COLOR_SPEC(text, NORMAL, normal);
  OUTPUT_COLOR_SPEC(base, NORMAL, normal);
  OUTPUT_COLOR_SPEC(text, PRELIGHT, prelight);
  OUTPUT_COLOR_SPEC(base, PRELIGHT, prelight);
  OUTPUT_COLOR_SPEC(text, ACTIVE, active);
  OUTPUT_COLOR_SPEC(base, ACTIVE, active);
  OUTPUT_COLOR_SPEC(text, SELECTED, selected);
  OUTPUT_COLOR_SPEC(base, SELECTED, selected);
  OUTPUT_COLOR_SPEC(text, INSENSITIVE, insensitive);
  OUTPUT_COLOR_SPEC(base, INSENSITIVE, insensitive);

  gtk_color_button_get_color(GTK_COLOR_BUTTON(text_normal_state_color_button),
			     &color);
  fprintf(file, "  GtkWidget::cursor_color=\"#%02x%02x%02x\"\n\n",
	  (color.red >> 8) & 0xff,
	  (color.green >> 8) & 0xff,
	  (color.blue >> 8) & 0xff);

  fprintf(file, "}\n\n");
  fprintf(file, "class \"GtkWidget\" style \"ochusha-common-style\"\n");

  fclose(file);
}


void
PreferencesUI::reload_anarchy_gtkrc()
{
  char *rc_file = controller.get_repository().find_file(OCHUSHA_ANARCHY_GTKRC);

  if (rc_file != NULL)
    {
      gtk_rc_parse(rc_file);
      gtk_rc_add_default_file(rc_file);
      free(rc_file);
    }

  gtk_widget_reset_rc_styles(GTK_WIDGET(controller.get_toplevel()));
}


void
PreferencesUI::setup_dialog()
{
  g_return_if_fail(dialog == NULL);
  if (a_bone_settings != NULL)
    {
      delete a_bone_settings;
      a_bone_settings = NULL;
    }

  dialog = gtk_dialog_new_with_buttons(_("Preferences of Ochusha"),
				       controller.get_toplevel(),
				       GTK_DIALOG_DESTROY_WITH_PARENT,
				       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				       GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
				       GTK_STOCK_OK, GTK_RESPONSE_OK,
				       NULL);
  gtk_dialog_set_default_response(GTK_DIALOG(dialog),
				  GTK_RESPONSE_OK);
  g_signal_connect(dialog, "response",
		   reinterpret_cast<GCallback>(response_cb), this);
  g_signal_connect(dialog, "destroy",
		   reinterpret_cast<GCallback>(gtk_widget_destroyed), &dialog);

  GtkWidget *notebook = gtk_notebook_new();
  gtk_widget_show(notebook);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
		     notebook, TRUE, TRUE, 0);

  GtkWidget *tmp_widget = create_general_settings_frame();
  gtk_widget_show(tmp_widget);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
			   gtk_label_new(_("General")));

  tmp_widget = create_list_view_settings_frame();
  gtk_widget_show(tmp_widget);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
			   gtk_label_new(_("List View")));

  tmp_widget = create_thread_view_settings_frame();
  gtk_widget_show(tmp_widget);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
			   gtk_label_new(_("Thread View")));

  tmp_widget = create_a_bone_settings_frame();
  a_bone_settings_frame = tmp_widget;
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
			   gtk_label_new(_("A Bone")));

  tmp_widget = create_image_view_settings_frame();
  gtk_widget_show(tmp_widget);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
			   gtk_label_new(_("Image View")));

  tmp_widget = create_my_bookmark_settings_frame();
  gtk_widget_show(tmp_widget);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
			   gtk_label_new(_("My Bookmark")));

  tmp_widget = create_network_settings_frame();
  gtk_widget_show(tmp_widget);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
			   gtk_label_new(_("Network")));

  tmp_widget = create_state_color_settings_frame();
  state_color_settings_frame = tmp_widget;
  if (tmp_widget != NULL)
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
			     gtk_label_new(_("Colors")));


  tmp_widget = gtk_check_button_new_with_label(_("Show all, odd, experimental, non-gentle, vulgar and minor detail settings"));
  g_signal_connect(tmp_widget, "toggled",
		   reinterpret_cast<GCallback>(show_all_toggled_cb), this);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_end(GTK_BOX(GTK_DIALOG(dialog)->vbox),
		   tmp_widget, FALSE, FALSE, 0);

  SessionClient &session = controller.get_session();
  const char *value = session.get_state("PreferencesUI::show_all");
  if (value != NULL && *value != '0')
    {
      // Τno_show_allץѥƥʤΤǡġ
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), FALSE);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), TRUE);
    }
  else
    {
      // Τno_show_allץѥƥʤΤǡġ
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), TRUE);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget), FALSE);
    }
  gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog),
				    GTK_RESPONSE_APPLY, FALSE);
}

}
