/* -*- c++ -*-
 * Copyright (c) 2003-2007 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: tab_label.h,v 1.12 2007/01/01 03:18:25 fuyu Exp $
 */

#ifndef _TAB_LABEL_H_
#define _TAB_LABEL_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/sigslot.h"
#include "libochushagtk/icon_label.h"
#include "libochushagtk/stock.h"

#include <gtk/gtk.h>

#include <list>


using namespace std;
namespace Ochusha
{
  class TabLabelIconSet;
  class TabLabelManager;

  class TabLabel : public has_slots
  {
    friend class TabLabelManager;

  private:
    const TabLabelIconSet &icon_set;
    OchushaIconLabel *label;
    bool in_error;


    TabLabel(const gchar *text, const TabLabelIconSet &icon_set);
    ~TabLabel();

    static void icon_label_destroy_cb(GtkWidget *widget, TabLabel *label);


  public:
    inline GtkWidget *get_widget()
    { return reinterpret_cast<GtkWidget *>(label); }
    static TabLabel *get_label(GtkWidget *widget);

    void access_started();
    void access_progressed();
    void access_finished();
    void access_failed();
    void access_terminated();

    inline bool is_error_state() const { return in_error; }
    void notice(bool show);
  };


  class TabLabelManager
  {
    friend class TabLabelIconSet;

  private:
    GdkPixbuf *done_addon;
    GdkPixbuf *error_addon;
    GdkPixbuf *notice_addon;
    GdkPixbuf *preparation_addon;
    GdkPixbuf *wait_addon;
    GdkPixbuf *progress_addons[OCHUSHA_NUMBER_OF_NET_PROGRESS_ADDONS];

    list<TabLabelIconSet *> icon_set_list;


  public:
    TabLabelManager();
    ~TabLabelManager();

    TabLabel *tab_label_new(const gchar *text, const gchar *stock_id);
  };


}


#endif	/* __cplusplus */

#endif	/* _TAB_LABEL_H_ */
