/* -*- c++ -*-
 * Copyright (c) 2003-2007 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: threadlist_view.h,v 1.20 2007/01/01 03:18:25 fuyu Exp $
 */

#ifndef _THREADLIST_VIEW_H_
#define _THREADLIST_VIEW_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/bulletin_board.h"
#include "libochusha/ref_count.h"

#include "selection_utils.h"

#include "libochushagtk/tree_selector_view.h"

#include <vector>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{


  class ThreadlistViewSelectionData : public SelectionData
  {
  private:
    GtkTreeSelection *selection;
    vector<BBSThread *> thread_list;

    static void accumulate_threads(GtkTreeModel *model, GtkTreePath *path,
				   GtkTreeIter *iter, gpointer data);
    static void changed_cb(GtkTreeSelection *selection,
			   ThreadlistViewSelectionData *data);


  protected:
    virtual const GtkTargetEntry *get_targets();
    virtual guint get_n_targets();

    virtual void get_data(GtkClipboard *clipboard, GtkSelectionData *data,
			  guint info);
    // 줬true֤顢delete this¹Ԥ롣
    virtual bool clear_data(GtkClipboard *clipboard);


  public:
    ThreadlistViewSelectionData(GtkTreeSelection *selection);
    virtual ~ThreadlistViewSelectionData();

    bool set_data(GtkSelectionData *data, guint info);

    SelectionData *create_proxy();
  };


  class Controller;
  class TabLabel;
  class ThreadlistRuleInfo;

  class ThreadlistView : public has_slots
  {
    friend class ThreadlistUI;

  private:
    Controller &controller;

    GtkTreeModel *model;
    GtkWidget *widget;			// OchushaTreeSelectorView

    ThreadlistViewSelectionData selection_data;

    GtkTreeViewColumn *weight_column;
    GtkTreeViewColumn *last_sort_column;

    GtkTreePath *last_event_path;
    gint press_start_x;
    gint press_start_y;

    guint press_start_x_root;
    guint press_start_y_root;
    guint press_start_button;
    guint press_start_state;

    guint popup_delay_timeout_id;


    // ɽ̵ʤȺΤǡġ
    smart_ptr<BulletinBoard> board;
    vector<smart_ptr<BBSThread> > shown_threads;

    const ThreadlistRuleInfo *rule_info;
    smart_ptr<ThreadlistIterator> list_iter;
    guint append_threads_id;
    guint number_of_threads;

    void setup_columns();

    static void column_set_mark(GtkTreeViewColumn *column,
				GtkCellRenderer *renderer,
				GtkTreeModel *model, GtkTreeIter *iter,
				gpointer data);
    static void column_set_text(GtkCellRenderer *renderer, BBSThread *thread,
				const gchar *text, gpointer data);
    static void column_set_board_name(GtkTreeViewColumn *column,
				      GtkCellRenderer *renderer,
				      GtkTreeModel *model, GtkTreeIter *iter,
				      gpointer view);
    static void column_set_title(GtkTreeViewColumn *column,
				 GtkCellRenderer *renderer,
				 GtkTreeModel *model, GtkTreeIter *iter,
				 gpointer view);
    static void column_set_rank(GtkTreeViewColumn *column,
				GtkCellRenderer *renderer,
				GtkTreeModel *model, GtkTreeIter *iter,
				gpointer view);
    static void column_set_n_res_verbose(GtkTreeViewColumn *column,
					 GtkCellRenderer *renderer,
					 GtkTreeModel *model, GtkTreeIter *iter,
					 gpointer view);
    static void column_set_n_res_verbose2(GtkTreeViewColumn *column,
					  GtkCellRenderer *renderer,
					  GtkTreeModel *model,
					  GtkTreeIter *iter,
					  gpointer view);
    static void column_set_n_res_avail(GtkTreeViewColumn *column,
				       GtkCellRenderer *renderer,
				       GtkTreeModel *model, GtkTreeIter *iter,
				       gpointer view);
    static void column_set_n_res_read(GtkTreeViewColumn *column,
				      GtkCellRenderer *renderer,
				      GtkTreeModel *model, GtkTreeIter *iter,
				      gpointer view);
    static void column_set_n_res_ungot(GtkTreeViewColumn *column,
				       GtkCellRenderer *renderer,
				       GtkTreeModel *model, GtkTreeIter *iter,
				       gpointer view);
    static void column_set_n_res_to_read(GtkTreeViewColumn *column,
					 GtkCellRenderer *renderer,
					 GtkTreeModel *model, GtkTreeIter *iter,
					 gpointer view);
    static void column_set_n_new_res(GtkTreeViewColumn *column,
				     GtkCellRenderer *renderer,
				     GtkTreeModel *model, GtkTreeIter *iter,
				     gpointer view);
    static void column_set_rank_diff(GtkTreeViewColumn *column,
				     GtkCellRenderer *renderer,
				     GtkTreeModel *model, GtkTreeIter *iter,
				     gpointer view);
    static void column_set_birthtime(GtkTreeViewColumn *column,
				     GtkCellRenderer *renderer,
				     GtkTreeModel *model, GtkTreeIter *iter,
				     gpointer view);
    static void column_set_last_modified(GtkTreeViewColumn *column,
					 GtkCellRenderer *renderer,
					 GtkTreeModel *model, GtkTreeIter *iter,
					 gpointer view);
    static void column_set_weight(GtkTreeViewColumn *column,
				  GtkCellRenderer *renderer,
				  GtkTreeModel *model, GtkTreeIter *iter,
				  gpointer view);
    static void column_set_search_weight(GtkTreeViewColumn *column,
					 GtkCellRenderer *renderer,
					 GtkTreeModel *model, GtkTreeIter *iter,
					 gpointer view);

    static void column_clicked_cb(GtkTreeViewColumn *column,
				  OchushaTreeSelectorView *widget);

    static void set_sort_column_id(GtkTreeViewColumn *column,
				   gint column_id, GtkSortType order,
				   GtkWidget *view);

    void restore_session(GtkTreeViewColumn *column, const char *name);
    void append_board_name_column();
    void append_title_column();
    void append_rank_column();
    void append_mark_column();
    void append_n_res_verbose_column();
    void append_n_res_verbose2_column();
    void append_n_res_avail_column();
    void append_n_res_read_column();
    void append_n_res_ungot_column();
    void append_n_res_to_read_column();
    void append_n_new_res_column();
    void append_rank_diff_column();
    void append_birthtime_column();
    void append_last_modified_column();
    void append_weight_column();
    void append_search_weight_column();

    // DnDݡȤμ¸Ƥ
    void initialize_for_drag_source();

    static void drag_data_get_cb(GtkWidget *widget,
				 GdkDragContext *drag_context,
				 GtkSelectionData *data,
				 guint info, guint time,
				 ThreadlistView *view);
    // ޤDnDϢ


    static void selection_changed_cb(GtkTreeSelection *selection,
				     ThreadlistView *view);
    static gboolean popup_menu_cb(GtkWidget *, ThreadlistView *view);
    static gboolean motion_notify_cb(GtkWidget *widget, GdkEventMotion *event,
				     ThreadlistView *view);
    static gboolean popup_delay_timeout_cb(ThreadlistView *view);

    void popup_context_menu(guint button, guint32 event_time);

    static gboolean row_button_press_cb(OchushaTreeSelectorView *widget,
					GtkTreePath *path,
					GtkTreeViewColumn *column,
					GdkEventButton *event,
					ThreadlistView *view);
    static gboolean row_button_release_cb(OchushaTreeSelectorView *widget,
					  GtkTreePath *path,
					  GtkTreeViewColumn *column,
					  GdkEventButton *event,
					  ThreadlistView *view);
    static void column_resized_cb(OchushaTreeSelectorView *widget,
				  GtkTreeViewColumn *column,
				  gboolean use_resized_width,
				  gint resized_width,
				  ThreadlistView *view);

    ThreadlistView(Controller &controller, BulletinBoard *board,
		   TabLabel *label);
    virtual ~ThreadlistView();

    static void widget_destroy_cb(GtkWidget *widget, ThreadlistView *view);


    void append_thread(BBSThread *thread);
    static gboolean append_threads_on_idle(ThreadlistView *view);
    void state_changed_cb();

    // ThreadlistUIǤΥݥåץåץ˥塼μ
    static BBSThread *get_thread(GtkTreeModel *model, GtkTreeIter *iter);


  public:
    // ThreadlistView֥Ȥget_widget()֤GtkWidgetȰϡ
    static GtkWidget *create(Controller &controller, BulletinBoard *board,
			     TabLabel *label);
    static void update(GtkWidget *widget, TabLabel *label, bool reload);
    static ThreadlistView *get_view(GtkWidget *widget);

    // ݥåץåץ˥塼actionѡ
    GtkTreeSelection *get_selection();
  };


}


#endif	/* __cplusplus */

#endif	/* _THREADLIST_VIEW_H_ */
