/* -*- c++ -*-
 * Copyright (c) 2003-2007 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: tree_ui.cc,v 1.39 2007/01/01 03:18:25 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "tree_ui.h"

#include "tree_ui_impl.h"

#include "controller.h"
#include "ui_constants.h"

#include "libochushagtk/misc_utils.h"

#include <algorithm>
#include <cassert>
#include <list>
#include <vector>


namespace Ochusha
{


TreeUI::TreeUI(Controller &controller)
  : controller(controller),
    action_group(create_action_group())
{
}


TreeUI::~TreeUI()
{
  g_object_unref(action_group);
}


GtkActionGroup *
TreeUI::create_action_group()
{
  static GtkActionEntry entries[] = {
    {
      "NewFolder", GTK_STOCK_DIRECTORY,
      N_("New _Folder"), NULL,
      N_("New Folder"),
      reinterpret_cast<GCallback>(new_folder_cb)
    },
    {
      "ToggleTreeItemHiddenState", NULL,
      N_("Toggle Hi_de"), NULL,
      N_("Toggle Hide"),
      reinterpret_cast<GCallback>(toggle_hide_cb)
    },
    {
      "HideTreeItem", NULL,
      N_("Hi_de"), NULL,
      N_("Hide"),
      reinterpret_cast<GCallback>(hide_cb)
    },
    {
      "KillTreeItem", GTK_STOCK_DELETE,
      N_("_Kill"), "",
      N_("Kill"),
      reinterpret_cast<GCallback>(kill_cb)
    },
    {
      "CopyTreeItemName", GTK_STOCK_COPY,
      N_("Copy _Name"), "",
      N_("Copy Name"),
      reinterpret_cast<GCallback>(copy_name_cb)
    },
    {
      "CopyTreeItemNameURL", GTK_STOCK_COPY,
      N_("Copy Name _and URL"), "",
      N_("Copy Name and URL"),
      reinterpret_cast<GCallback>(copy_name_url_cb)
    },
    {
      "TreeItemProperties", GTK_STOCK_PROPERTIES,
      N_("_Properties"), NULL,
      N_("Properties"),
      reinterpret_cast<GCallback>(properties_cb)
    }
  };

  GtkActionGroup *action_group = gtk_action_group_new("TreeUIPopupActions");

  gtk_action_group_set_translation_domain(action_group, PACKAGE_NAME);
  gtk_action_group_add_actions(action_group, entries,
			       G_N_ELEMENTS(entries),
			       controller.get_ui_manager());

  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     action_group, 0);
  return action_group;
}


inline static TreeUIImpl *
get_tree_ui_impl(GtkUIManager *ui_manager)
{
  return static_cast<TreeUIImpl *>(ochusha_ui_manager_get_popup_data(ui_manager));
}


void
TreeUI::new_folder_cb(GtkAction *action, GtkUIManager *ui_manager)
{
  TreeUIImpl *impl = get_tree_ui_impl(ui_manager);
  impl->new_folder_selection();
}


void
TreeUI::toggle_hide_cb(GtkAction *action, GtkUIManager *ui_manager)
{
  TreeUIImpl *impl = get_tree_ui_impl(ui_manager);
  impl->toggle_hide_selection();
}


void
TreeUI::hide_cb(GtkAction *action, GtkUIManager *ui_manager)
{
  TreeUIImpl *impl = get_tree_ui_impl(ui_manager);
  impl->hide_selection();
}


void
TreeUI::kill_cb(GtkAction *action, GtkUIManager *ui_manager)
{
  TreeUIImpl *impl = get_tree_ui_impl(ui_manager);
  impl->kill_selection();
}


void
TreeUI::copy_name_cb(GtkAction *action, GtkUIManager *ui_manager)
{
  TreeUIImpl *impl = get_tree_ui_impl(ui_manager);
  impl->copy_name_selection();
}


void
TreeUI::copy_name_url_cb(GtkAction *action, GtkUIManager *ui_manager)
{
  TreeUIImpl *impl = get_tree_ui_impl(ui_manager);
  impl->copy_name_url_selection();
}


void
TreeUI::properties_cb(GtkAction *action, GtkUIManager *ui_manager)
{
  TreeUIImpl *impl = get_tree_ui_impl(ui_manager);
  impl->open_properties_selection();
}


}
